(function(S,T,U,V,P,W,B,L,C,E,D,Q,X,F,Y,G,s,H,N,O,I,M,Z,R,$,aa,ba,ca,da,ea,K,fa,ga,ha,ia,ja,ka,la,ma,na,oa,pa,qa,ra,sa,ta,ua,va,wa,xa,ya,za,Aa,Ba,Ca,Da,Ea,Fa,h,n){B("JS");C(["JU.BS","$.V3"],"JS.VTemp",null,function(){c$=D(function(){this.bsTemp=this.vNorm4=this.vNorm3=this.vNorm2=this.vTemp2=this.vTemp1=this.vB=this.vA=this.vTemp=null;E(this,arguments)},JS,"VTemp");I(c$,function(){this.vTemp=new JU.V3;this.vA=new JU.V3;this.vB=new JU.V3;this.vTemp1=new JU.V3;this.vTemp2=new JU.V3;this.vNorm2=new JU.V3;
this.vNorm3=new JU.V3;this.vNorm4=new JU.V3;this.bsTemp=new JU.BS})});B("J.api");R(J.api,"SmilesMatcherInterface");B("JS");C(["J.api.SmilesMatcherInterface"],"JS.SmilesMatcher","JU.AU $.BS $.PT JS.InvalidSmilesException $.SmilesAtom $.SmilesBond $.SmilesGenerator $.SmilesParser $.SmilesSearch JU.BSUtil $.Elements $.Logger $.Node $.Point3fi".split(" "),function(){c$=O(JS,"SmilesMatcher",null,J.api.SmilesMatcherInterface);n(c$,"getLastException",function(){return JS.InvalidSmilesException.getLastError()});
n(c$,"getMolecularFormula",function(a,b){JS.InvalidSmilesException.clear();var c=JS.SmilesParser.newSearch("/nostereo/"+a,b,!0);c.createTopoMap(null);c.nodes=c.targetAtoms;return c.getMolecularFormula(!b,null,!1)},"~S,~B");n(c$,"getSmiles",function(a,b,c,d,e){JS.InvalidSmilesException.clear();return(new JS.SmilesGenerator).getSmiles(this,a,b,c,d,e)},"~A,~N,JU.BS,~S,~N");n(c$,"areEqual",function(a,b){JS.InvalidSmilesException.clear();var c=this.findPriv(a,JS.SmilesParser.newSearch(b,!1,!0),(0<=a.indexOf("*")?
2:1)|8,2);return null==c?-1:c.length},"~S,~S");h(c$,"areEqualTest",function(a,b){var c=this.findPriv(a,b,9,2);return null!=c&&1==c.length},"~S,JS.SmilesSearch");n(c$,"find",function(a,b,c){JS.InvalidSmilesException.clear();b=JS.SmilesParser.cleanPattern(b);a=JS.SmilesParser.cleanPattern(a);b=JS.SmilesParser.newSearch(b,!1,!0);a=this.findPriv(a,b,c,3);for(c=a.length;0<=--c;)for(var d=a[c],e=d.length;0<=--e;)d[e]=b.targetAtoms[d[e]].mapIndex;return a},"~S,~S,~N");n(c$,"getRelationship",function(a,b){if(null==
a||null==b||0==a.length||0==b.length)return"";var c=this.getMolecularFormula(a,!1),d=this.getMolecularFormula(b,!1);if(!c.equals(d))return"none";var e,c=JU.PT.countChar(JU.PT.rep(a,"@@","@"),"@"),d=JU.PT.countChar(JU.PT.rep(b,"@@","@"),"@");e=c==d&&0<this.areEqual(b,a);if(!e){e=a+b;if(0<=e.indexOf("/")||0<=e.indexOf("\\")||0<=e.indexOf("@")){if(c==d&&(0<c&&0>e.indexOf("@SP"))&&(e=0<this.areEqual("/invertstereo/"+b,a)))return"enantiomers";if(e=0<this.areEqual("/nostereo/"+b,a))return c==d?"diastereomers":
"ambiguous stereochemistry!"}return"constitutional isomers"}return"identical"},"~S,~S");n(c$,"reverseChirality",function(a){a=JU.PT.rep(a,"@@","!@");a=JU.PT.rep(a,"@","@@");return a=JU.PT.rep(a,"!@@","@")},"~S");n(c$,"getSubstructureSet",function(a,b,c,d,e){return this.matchPriv(a,b,c,d,null,!0,e|JS.SmilesParser.getFlags(a),1)},"~S,~A,~N,JU.BS,~N");n(c$,"getMMFF94AtomTypes",function(a,b,c,d,e,f){JS.InvalidSmilesException.clear();var g=new JS.SmilesParser(!0,!0),j=null,j=g.parse("");j.exitFirstMatch=
!1;j.targetAtoms=b;j.targetAtomCount=Math.abs(c);j.setSelected(d);j.flags=770;j.getRingData(f,!0,!0);j.asVector=!1;j.subSearches=Array(1);j.getSelections();b=new JU.BS;for(d=0;d<a.length;d++)if(null==a[d]||0==a[d].length||a[d].startsWith("#"))e.addLast(null);else if(j.clear(),j.subSearches[0]=g.getSubsearch(j,JS.SmilesParser.cleanPattern(a[d]),770),f=JU.BSUtil.copy(j.search()),e.addLast(f),b.or(f),b.cardinality()==c)break},"~A,~A,~N,JU.BS,JU.Lst,~A");n(c$,"getSubstructureSetArray",function(a,b,c,
d,e,f){return this.matchPriv(a,b,c,d,e,!0,f,2)},"~S,~A,~N,JU.BS,JU.BS,~N");h(c$,"getAtropisomerKeys",function(a,b,c,d,e,f){return this.matchPriv(a,b,c,d,e,!1,f,4)},"~S,~A,~N,JU.BS,JU.BS,~N");n(c$,"polyhedronToSmiles",function(a,b,c,d,e,f){for(var g=Array(c),j=0;j<c;j++){g[j]=new JS.SmilesAtom;var l=null==d?null:d[j];L(l,JU.Node)?(g[j].elementNumber=l.getElementNumber(),g[j].bioAtomName=l.getAtomName(),g[j].atomNumber=l.getAtomNumber(),g[j].setT(l)):g[j].elementNumber=L(l,JU.Point3fi)?l.sD:-2;g[j].index=
j}l=0;for(j=b.length;0<=--j;)for(var t=b[j],k=t.length,m,h,p=k;0<=--p;)if(!((m=t[p])>=c||(h=t[(p+1)%k])>=c)&&null==g[m].getBondTo(g[h]))(new JS.SmilesBond(g[m],g[h],1,!1)).index=l++;for(j=0;j<c;j++)if(k=g[j].bondCount,0==k||k!=g[j].bonds.length)g[j].bonds=JU.AU.arrayCopyObject(g[j].bonds,k);b=null;b=new JS.SmilesGenerator;null!=d&&(b.polySmilesCenter=a);JS.InvalidSmilesException.clear();b=b.getSmiles(this,g,c,JU.BSUtil.newBitSet2(0,c),null,e|4144);65536==(e&65536)&&(b="//* "+a+" *//\t["+JU.Elements.elementSymbolFromNumber(a.getElementNumber())+
"@PH"+c+(null==f?"":"/"+f+"/")+"]."+b);return b},"JU.Node,~A,~N,~A,~N,~S");n(c$,"getCorrelationMaps",function(a,b,c,d,e){return this.matchPriv(a,b,c,d,null,!0,e,3)},"~S,~A,~N,JU.BS,~N");h(c$,"findPriv",function(a,b,c,d){var e=new JU.BS;b.setFlags(b.flags|JS.SmilesParser.getFlags(a));b.createTopoMap(e);return this.matchPriv(a,b.targetAtoms,-b.targetAtoms.length,null,e,e.isEmpty(),c,d)},"~S,JS.SmilesSearch,~N,~N");h(c$,"matchPriv",function(a,b,c,d,e,f,g,j){JS.InvalidSmilesException.clear();try{var l=
2==(g&2),t=JS.SmilesParser.newSearch(a,l,!1);!l&&!t.patternAromatic&&(null==e&&(e=new JU.BS),JS.SmilesSearch.normalizeAromaticity(t.patternAtoms,e,t.flags),t.isNormalized=!0);t.targetAtoms=b;t.targetAtomCount=Math.abs(c);0>c&&(t.haveTopo=!0);if(0!=c&&(null==d||!d.isEmpty())){var k=!L(b[0],JS.SmilesAtom);t.setSelected(d);t.getSelections();f||(t.bsAromatic=e);t.setRingData(null,null,k||f);t.exitFirstMatch=8==(g&8);t.mapUnique=128==(g&128)}switch(j){case 1:return t.asVector=!1,t.search();case 2:t.asVector=
!0;var m=t.search();return m.toArray(Array(m.size()));case 4:return t.exitFirstMatch=!0,t.setAtropicity=!0,t.search(),t.atropKeys;case 3:t.getMaps=!0;t.setFlags(g|t.flags);var h=t.search();return h.toArray(JU.AU.newInt2(h.size()))}}catch(p){if(G(p,Exception))throw JU.Logger.debugging&&p.printStackTrace(),null==JS.InvalidSmilesException.getLastError()&&JS.InvalidSmilesException.clear(),new JS.InvalidSmilesException(JS.InvalidSmilesException.getLastError());throw p;}return null},"~S,~A,~N,JU.BS,JU.BS,~B,~N,~N");
n(c$,"cleanSmiles",function(a){return JS.SmilesParser.cleanPattern(a)},"~S");n(c$,"getMapForJME",function(a,b,c){var d=new JS.SmilesSearch,e=JU.PT.getTokens(a);a=JU.PT.parseInt(e[0]);for(var f=JU.PT.parseInt(e[1]),g=2,j=0;j<a;j++,g+=3){var l=e[g],t=d.addAtom(),k=l.indexOf("+"),m=0;if(0<=k)m=k==l.length-1?1:JU.PT.parseInt(l.substring(k+1));else if(0<=(k=l.indexOf("-")))m=JU.PT.parseInt(l.substring(k));t.setCharge(m);t.setSymbol(0>k?l:l.substring(0,k))}for(j=0;j<f;j++){t=JU.PT.parseInt(e[g++])-1;k=
JU.PT.parseInt(e[g++])-1;l=JU.PT.parseInt(e[g++]);t=d.patternAtoms[t];k=d.patternAtoms[k];m=1;switch(l){case 2:m=2;break;case 3:m=3}(new JS.SmilesBond(t,k,m,!1)).index=j}e="";try{d.isSmarts=!0;d.set();var h=JU.BSUtil.newBitSet2(0,a),e=this.getSmiles(d.patternAtoms,d.ac,h,null,34),p=this.getCorrelationMaps(e,d.patternAtoms,a,h,42),q=this.getCorrelationMaps(e,b,c.cardinality(),c,42);return K(-1,[p[0],q[0]])}catch(r){if(G(r,Exception))r.printStackTrace();else throw r;}return null},"~S,~A,JU.BS");H(c$,
"MODE_BITSET",1,"MODE_ARRAY",2,"MODE_MAP",3,"MODE_ATROP",4)});B("JS");C(["java.lang.Exception"],"JS.InvalidSmilesException",null,function(){c$=O(JS,"InvalidSmilesException",Exception);c$.getLastError=h(c$,"getLastError",function(){return JS.InvalidSmilesException.lastError});c$.clear=h(c$,"clear",function(){JS.InvalidSmilesException.lastError=null});n(c$,"getMessage",function(){return JS.InvalidSmilesException.lastError});F(c$,function(a){M(this,JS.InvalidSmilesException,[a]);JS.InvalidSmilesException.lastError=
a.startsWith("Jmol SMILES")?a:"Jmol SMILES Exception: "+a},"~S");H(c$,"lastError",null)});B("JS");C(["JU.JmolMolecule","JU.BS","$.Lst"],"JS.SmilesSearch","java.lang.Float java.util.Hashtable JU.AU $.SB JS.InvalidSmilesException $.SmilesAromatic $.SmilesAtom $.SmilesBond $.SmilesMeasure $.SmilesParser $.VTemp JU.BSUtil $.Edge $.Logger".split(" "),function(){c$=D(function(){this.isSmarts=!1;this.targetAtoms=this.patternAtoms=this.pattern=this.top=null;this.targetAtomCount=0;this.v=this.bsSelected=null;
this.patternBioSequence=this.isTopology=this.haveTopo=this.patternAromatic=this.setAtropicity=this.groupByModel=this.exitFirstMatch=this.invertStereochemistry=this.ignoreStereochemistry=this.ignoreAtomClass=this.noAromatic=this.aromaticDouble=this.aromaticPlanar=this.aromaticStrict=this.aromaticOpen=!1;this.subSearches=null;this.haveBondStereochemistry=this.haveSelected=!1;this.stereo=null;this.needRingData=!1;this.needAromatic=!0;this.needRingMemberships=!1;this.nDouble=0;this.ringDataMax=-2147483648;
this.ringSets=null;this.ringCount=0;this.measures=null;this.flags=0;this.lastChainAtom=this.atropKeys=this.bsAromatic6=this.bsAromatic5=this.bsAromatic=null;this.isRingCheck=this.isSilent=this.haveComponents=this.isNormalized=this.getMaps=this.asVector=!1;this.selectedAtomCount=0;this.htNested=this.bsFound=this.ringConnections=this.ringCounts=this.ringData=null;this.nNested=0;this.bsCheck=this.bsReturn=this.uniqueList=this.vReturn=this.nestedBond=null;this.mapUnique=!1;this.bsAromaticRings=null;E(this,
arguments)},JS,"SmilesSearch",JU.JmolMolecule);I(c$,function(){this.patternAtoms=Array(16);this.measures=new JU.Lst;this.bsAromatic=new JU.BS;this.bsAromatic5=new JU.BS;this.bsAromatic6=new JU.BS;this.bsFound=new JU.BS;this.bsReturn=new JU.BS});F(c$,function(){M(this,JS.SmilesSearch,[]);this.top=this;this.v=new JS.VTemp});h(c$,"setTop",function(a){for(;a.top!==a;)a=a.top;this.top=a},"JS.SmilesSearch");c$.addFlags=h(c$,"addFlags",function(a,b){0<=b.indexOf("OPEN")&&(a|=5);0<=b.indexOf("BIO")&&(a|=
1048576);0<=b.indexOf("HYDROGEN")&&(a|=4096);0<=b.indexOf("FIRSTMATCHONLY")&&(a|=8);0<=b.indexOf("STRICT")&&(a|=256);0<=b.indexOf("PLANAR")&&(a|=1024);if(0<=b.indexOf("NOAROMATIC")||0<=b.indexOf("NONAROMATIC"))a|=16;0<=b.indexOf("AROMATICDOUBLE")&&(a|=512);0<=b.indexOf("AROMATICDEFINED")&&(a|=128);0<=b.indexOf("MMFF94")&&(a|=768);0<=b.indexOf("TOPOLOGY")&&(a|=8192);0<=b.indexOf("NOATOMCLASS")&&(a|=2048);0<=b.indexOf("NOSTEREO")?a|=32:0<=b.indexOf("INVERTSTEREO")&&(a=0!=(a&64)?a&-65:a|64);0<=b.indexOf("ATOMCOMMENT")&&
(a|=131072);0<=b.indexOf("GROUPBYMODEL")&&(a|=67108864);1048576==(a&1048576)&&(0<=b.indexOf("NOCOMMENT")&&(a|=34603008),0<=b.indexOf("UNMATCHED")&&(a|=3145728),0<=b.indexOf("COVALENT")&&(a|=5242880),0<=b.indexOf("HBOND")&&(a|=9437184));return a},"~N,~S");h(c$,"setFlags",function(a){this.flags=a;this.exitFirstMatch=(new Boolean(this.exitFirstMatch|8==(a&8))).valueOf();this.aromaticOpen=5==(a&5);this.aromaticDouble=512==(a&512);this.aromaticStrict=256==(a&256);this.aromaticPlanar=1024==(a&1024);this.groupByModel=
67108864==(a&67108864);this.noAromatic=16==(a&16);this.ignoreAtomClass=2048==(a&2048);this.ignoreStereochemistry=32==(a&32);this.invertStereochemistry=!this.ignoreStereochemistry&&64==(a&64)},"~N");h(c$,"set",function(){this.patternAtoms.length>this.ac&&(this.patternAtoms=JU.AU.arrayCopyObject(this.patternAtoms,this.ac));this.nodes=this.patternAtoms;this.isTopology=!0;this.patternAromatic=!1;this.patternBioSequence=!0;for(var a=this.ac;0<=--a;){var b=this.patternAtoms[a];this.isTopology&&b.isDefined()&&
(this.isTopology=!1);b.isBioResidue||(this.patternBioSequence=!1);b.isAromatic&&(this.patternAromatic=!0);b.setBondArray();if(!this.isSmarts&&"\x00"==b.bioType&&!b.setHydrogenCount())throw new JS.InvalidSmilesException("unbracketed atoms must be one of: B, C, N, O, P, S, F, Cl, Br, I, *,");}if(this.haveComponents)for(a=this.ac;0<=--a;)for(var b=this.patternAtoms[a],c=b.bonds,d=b.component,e=b.bondCount;0<=--e;){var f=c[e],g;if(f.isConnection&&f.atom2===b&&(g=f.atom1.component)!=d)for(f=this.ac;0<=
--f;)this.patternAtoms[f].component==d&&(this.patternAtoms[f].component=g)}});h(c$,"setSelected",function(a){null==a&&(a=JU.BS.newN(this.targetAtomCount),a.setBits(0,this.targetAtomCount));this.bsSelected=a},"JU.BS");h(c$,"addAtom",function(){return this.appendAtom(new JS.SmilesAtom)});h(c$,"appendAtom",function(a){this.ac>=this.patternAtoms.length&&(this.patternAtoms=JU.AU.doubleLength(this.patternAtoms));return this.patternAtoms[this.ac]=a.setIndex(this.ac++)},"JS.SmilesAtom");h(c$,"addNested",
function(a){null==this.htNested&&(this.htNested=new java.util.Hashtable);this.setNested(++this.nNested,a);return this.nNested},"~S");h(c$,"clear",function(){this.bsReturn.clearAll();this.nNested=0;this.nestedBond=this.htNested=null;this.clearBsFound(-1)});h(c$,"clearBsFound",function(a){0>a?null==this.bsCheck&&this.bsFound.clearAll():this.bsFound.clear(a)},"~N");h(c$,"setNested",function(a,b){this.top.htNested.put("_"+a,b)},"~N,~O");h(c$,"getNested",function(a){return this.top.htNested.get("_"+a)},
"~N");h(c$,"getMissingHydrogenCount",function(){for(var a=0,b,c=0;c<this.ac;c++)if(0<=(b=this.patternAtoms[c].explicitHydrogenCount))a+=b;return a});h(c$,"setRingData",function(a,b,c){if(this.isTopology||this.patternBioSequence)this.needAromatic=!1;this.needAromatic&&(this.needRingData=!0);this.needAromatic=(new Boolean(this.needAromatic&(new Boolean(null==a&16!=(this.flags&16))).valueOf())).valueOf();if(!this.needAromatic&&(this.bsAromatic.clearAll(),null!=a&&this.bsAromatic.or(a),!this.needRingMemberships&&
!this.needRingData))return;this.getRingData(b,this.needRingData,c)},"JU.BS,~A,~B");h(c$,"getRingData",function(a,b,c){var d=this.aromaticStrict||!this.aromaticOpen&&!this.aromaticPlanar,e=this.aromaticOpen&&!this.aromaticStrict,f=!d?0:768==(this.flags&768)?2:1,g=0==f,j=128==(this.flags&128),l=this.needAromatic&&c&&(d||e),t=null==a?new JU.Lst:a[3]=new JU.Lst,k=l?new JU.Lst:null,m=l?s(this.targetAtomCount,0):null;j&&this.needAromatic&&(JS.SmilesAromatic.checkAromaticDefined(this.targetAtoms,this.bsSelected,
this.bsAromatic),f=0);var h=this.targetAtomCount,p=0==h||L(this.targetAtoms[0],JS.SmilesAtom);0>this.ringDataMax&&(this.ringDataMax=8);0<f&&6>this.ringDataMax&&(this.ringDataMax=6);b&&(this.ringCounts=s(h,0),this.ringConnections=s(this.targetAtomCount,0),this.ringData=Array(this.ringDataMax+1));this.ringSets=new JU.Lst;for(var q="****",r=this.ringDataMax;q.length<r;)q+=q;for(var u=3;u<=r;u++)if(!(u>h)){var n="*1"+q.substring(0,u-2)+"*1",n=JS.SmilesParser.newSearch(n,!0,!0),n=this.subsearch(n,2);if(null!=
a&&5>=u){for(var v=new JU.Lst,x=n.size();0<=--x;)v.addLast(n.get(x));a[u-3]=v}if(0!=n.size()&&(this.needAromatic&&(!j&&4<=u&&7>=u)&&JS.SmilesAromatic.setAromatic(u,this.targetAtoms,this.bsSelected,n,this.bsAromatic,f,e,p,g,this.v,t,k,m,c),b)){this.ringData[u]=new JU.BS;for(v=n.size();0<=--v;){var w=n.get(v);this.ringData[u].or(w);for(x=w.nextSetBit(0);0<=x;x=w.nextSetBit(x+1))this.ringCounts[x]++}}}if(this.needAromatic){l&&JS.SmilesAromatic.finalizeAromatic(this.targetAtoms,this.bsAromatic,t,k,m,
e,d);this.bsAromatic5.clearAll();this.bsAromatic6.clearAll();for(u=t.size();0<=--u;)switch(a=t.get(u),a.and(this.bsAromatic),a.cardinality()){case 5:this.bsAromatic5.or(a);break;case 6:this.bsAromatic6.or(a)}}if(b)for(u=this.bsSelected.nextSetBit(0);0<=u;u=this.bsSelected.nextSetBit(u+1))if(b=this.targetAtoms[u],t=b.getEdges(),null!=t)for(v=t.length;0<=--v;)0<this.ringCounts[b.getBondedAtomIndex(v)]&&this.ringConnections[u]++},"~A,~B,~B");h(c$,"subsearch",function(a,b){a.ringSets=this.ringSets;a.mapUnique=
this.mapUnique;a.targetAtoms=this.targetAtoms;a.targetAtomCount=this.targetAtomCount;a.bsSelected=this.bsSelected;a.htNested=this.htNested;a.haveTopo=this.haveTopo;a.bsCheck=this.bsCheck;a.isSmarts=!0;a.bsAromatic=this.bsAromatic;a.bsAromatic5=this.bsAromatic5;a.bsAromatic6=this.bsAromatic6;a.ringData=this.ringData;a.ringCounts=this.ringCounts;a.ringConnections=this.ringConnections;switch(b){case 1:a.exitFirstMatch=!1;break;case 2:a.isRingCheck=!0;a.isSilent=!0;a.asVector=!0;break;case 3:a.ignoreAtomClass=
this.ignoreAtomClass,a.aromaticDouble=this.aromaticDouble,a.haveSelected=this.haveSelected,a.exitFirstMatch=this.exitFirstMatch,a.getMaps=this.getMaps,a.asVector=this.asVector,a.vReturn=this.vReturn,a.bsReturn=this.bsReturn,a.haveBondStereochemistry=this.haveBondStereochemistry}return a.search2(1==b)},"JS.SmilesSearch,~N");h(c$,"search",function(){return this.search2(!1)});h(c$,"search2",function(a){this.setFlags(this.flags);!this.isRingCheck&&(JU.Logger.debugging&&!this.isSilent)&&JU.Logger.debug("SmilesSearch processing "+
this.pattern);if(null==this.vReturn&&(this.asVector||this.getMaps))this.vReturn=new JU.Lst;null==this.bsSelected&&(this.bsSelected=JU.BS.newN(this.targetAtomCount),this.bsSelected.setBits(0,this.targetAtomCount));this.selectedAtomCount=this.bsSelected.cardinality();if(null!=this.subSearches)for(a=0;a<this.subSearches.length&&!(null!=this.subSearches[a]&&(this.subsearch(this.subSearches[a],3),this.exitFirstMatch&&(null==this.vReturn?0<=this.bsReturn.nextSetBit(0):0<this.vReturn.size())));a++);else 0<
this.ac&&(null==this.nestedBond?this.clearBsFound(-1):this.bsReturn.clearAll(),this.nextPatternAtom(-1,-1,a,-1));return this.asVector||this.getMaps?this.vReturn:this.bsReturn},"~B");h(c$,"nextPatternAtom",function(a,b,c,d){var e,f;if(++a<this.ac){var g=this.patternAtoms[a],j=0<=b?g.getBondTo(null):0==a?this.nestedBond:null;if(null==j){b=JU.BSUtil.copy(this.bsFound);var l=JU.BSUtil.copy(this.bsFound);if(0<=g.notBondedIndex)if(e=this.patternAtoms[g.notBondedIndex],j=e.getMatchingAtom(),e.isBioAtom)e=
j.getOffsetResidueAtom("\x00",1),0<=e&&b.set(e),e=j.getOffsetResidueAtom("\x00",-1),0<=e&&b.set(e);else{f=j.getEdges();for(e=0;e<f.length;e++)b.set(f[e].getOtherAtomNode(j).getIndex())}f=g.isBioAtomWild;e=this.bsSelected.nextSetBit(0);e=f&&0<=e?this.targetAtoms[e].getOffsetResidueAtom("\x00",e):e;for(var t,k=(0<a?this.patternAtoms[a-1]:g).component,m=g.component,h=this.haveComponents&&-2147483648!=m,j=e;0<=j;j=this.bsSelected.nextSetBit(j+1)){if(!b.get(j)&&!this.bsFound.get(j)){e=this.targetAtoms[j];
if(h&&!this.isRingCheck&&(d=this.groupByModel?e.getModelIndex():e.getMoleculeNumber(!1),t=0<a?this.patternAtoms[a-1].matchingComponent:d,k==m!=(t==d)))continue;if(!this.nextTargetAtom(g,e,a,j,c,d))return!1}f&&(e=this.targetAtoms[j].getOffsetResidueAtom(g.bioAtomName,1),0<=e&&(j=e-1))}this.bsFound=l;return!0}e=j.atom1.getMatchingAtom();switch(j.order){case 96:j=e.getOffsetResidueAtom(g.bioAtomName,1);if(0<=j){b=JU.BSUtil.copy(this.bsFound);e.getGroupBits(this.bsFound);if(this.doCheckAtom(j)&&!this.nextTargetAtom(g,
this.targetAtoms[j],a,j,c,d))return!1;this.bsFound=b}return!0;case 112:f=new JU.Lst;e.getCrossLinkVector(f,!0,!0);b=JU.BSUtil.copy(this.bsFound);e.getGroupBits(this.bsFound);for(j=2;j<f.size();j+=3)if(l=f.get(j).intValue(),this.doCheckAtom(l)&&!this.nextTargetAtom(g,this.targetAtoms[l],a,l,c,d))return!1;this.bsFound=b;return!0}f=e.getEdges();if(null!=f)for(j=0;j<f.length;j++)if(l=e.getBondedAtomIndex(j),this.doCheckAtom(l)&&!this.nextTargetAtom(g,this.targetAtoms[l],a,l,c,d))return!1;this.clearBsFound(b);
return!0}if(!this.ignoreStereochemistry&&!this.isRingCheck&&!this.checkStereochemistry())return!0;b=new JU.BS;for(j=d=0;j<this.ac;j++)if(g=this.patternAtoms[j].getMatchingAtomIndex(),c||!this.top.haveSelected||this.patternAtoms[j].selected){d++;b.set(g);this.patternAtoms[j].isBioAtomWild&&this.targetAtoms[g].getGroupBits(b);if(c)break;if(!this.isSmarts&&!this.setAtropicity&&0<this.patternAtoms[j].explicitHydrogenCount){g=this.targetAtoms[g];e=0;for(f=g.getEdges().length;e<f;e++)l=g.getBondedAtomIndex(e),
1==this.targetAtoms[l].getElementNumber()&&b.set(l)}}if(!this.isSmarts&&b.cardinality()!=this.selectedAtomCount)return!0;if(null!=this.bsCheck)if(c){this.bsCheck.clearAll();for(j=0;j<this.ac;j++)this.bsCheck.set(this.patternAtoms[j].getMatchingAtomIndex());if(this.bsCheck.cardinality()!=this.ac)return!0}else if(b.cardinality()!=this.ac)return!0;this.bsReturn.or(b);if(this.getMaps){if(this.mapUnique){null==this.uniqueList&&(this.uniqueList=new JU.Lst);for(j=this.uniqueList.size();0<=--j;)if(this.uniqueList.get(j).equals(b))return!0;
this.uniqueList.addLast(b)}a=s(d,0);for(b=j=0;j<this.ac;j++)if(c||!this.top.haveSelected||this.patternAtoms[j].selected)a[b++]=this.patternAtoms[j].getMatchingAtomIndex();this.vReturn.addLast(a);return!this.exitFirstMatch}if(this.asVector){c=!0;for(j=this.vReturn.size();0<=--j&&c;)c=!this.vReturn.get(j).equals(b);if(!c)return!0;this.vReturn.addLast(b)}if(this.isRingCheck){c=new JU.BS;for(e=3*a+2;--e>a;)c.set(this.patternAtoms[(e<=2*a?2*a-e+1:e-1)%a].getMatchingAtomIndex());this.ringSets.addLast(c);
return!0}return this.exitFirstMatch?!1:b.cardinality()!=this.selectedAtomCount},"~N,~N,~B,~N");h(c$,"doCheckAtom",function(a){return this.bsSelected.get(a)&&!this.bsFound.get(a)},"~N");h(c$,"nextTargetAtom",function(a,b,c,d,e,f){if(!this.isRingCheck&&!this.isTopology)if(null==a.subAtoms){if(!this.checkPrimitiveAtom(a,d))return!0}else if(a.isAND)for(var g=0;g<a.nSubAtoms;g++){if(!this.checkPrimitiveAtom(a.subAtoms[g],d))return!0}else{for(g=0;g<a.nSubAtoms;g++)if(!this.nextTargetAtom(a.subAtoms[g],
b,c,d,e,f))return!1;return!0}b=b.getEdges();for(g=a.getBondCount();0<=--g;){var j=a.getBond(g);if(j.getAtomIndex2()==a.index){var l=j.atom1,t=l.getMatchingAtomIndex();switch(j.order){case 96:case 112:if(!this.checkMatchBond(a,l,j,d,t,null))return!0;break;default:for(var k=0,m=null;k<b.length&&(!(m=b[k]).isCovalent()||!(m.getAtomIndex1()==t||m.getAtomIndex2()==t));k++);if(k==b.length||!this.checkMatchBond(a,l,j,d,t,m))return!0}}}a=this.patternAtoms[a.index];a.setMatchingAtom(this.targetAtoms[d],d);
a.matchingComponent=f;if(JU.Logger.debugging&&!this.isRingCheck){for(g=0;g<=c;g++)JU.Logger.debug("pattern atoms "+this.patternAtoms[g]+" "+this.patternAtoms[g].matchingComponent);JU.Logger.debug("--ss--")}this.bsFound.set(d);if(!this.nextPatternAtom(c,d,e,f))return!1;0<=d&&this.clearBsFound(d);return!0},"JS.SmilesAtom,JU.Node,~N,~N,~B,~N");h(c$,"checkPrimitiveAtom",function(a,b){if(0<a.nSubAtoms){for(var c=0;c<a.nSubAtoms;c++)if(this.checkPrimitiveAtom(a.subAtoms[c],b))return!0;return!1}for(var d=
this.targetAtoms[b],c=a.not;;){if(0<a.iNested){c=this.getNested(a.iNested);L(c,JS.SmilesSearch)&&(a.isBioAtom&&(c.nestedBond=a.getBondTo(null)),c=this.subsearch(c,1),null==c&&(c=new JU.BS),a.isBioAtom||this.setNested(a.iNested,c));c=a.not!=c.get(b);break}var e=d.getElementNumber(),f=a.elementNumber;if(0<=e&&0<=f&&f!=e)break;if(a.isBioResidue){if(null!=a.bioAtomName&&(a.isLeadAtom()?!d.isLeadAtom():!a.bioAtomName.equals(d.getAtomName().toUpperCase())))break;if(null!=a.residueName&&!a.residueName.equals(d.getGroup3(!1).toUpperCase()))break;
if(-2147483648!=a.residueNumber&&a.residueNumber!=d.getResno())break;if("\x00"!=a.insCode&&a.insCode!=d.getInsertionCode())break;if(null!=a.residueChar||-2==a.elementNumber){var e=d.getBioSmilesType(),g=a.getBioSmilesType(),j=!0,f=!1;switch(g){case "\x00":case "*":j=!0;break;case "n":j="r"==e||"c"==e;f=!0;break;case "r":case "c":f=!0;default:j=e==g}if(!j)break;j=d.getGroup1("\x00").toUpperCase();g=null==a.residueChar?"*":a.residueChar.charAt(0);j=g==j.charAt(0);switch(g){case "*":j=!0;break;case "N":j=
f?"r"==e||"c"==e:j;break;case "R":j=f?d.isPurine():j;break;case "Y":j=f?d.isPyrimidine():j}if(!j)break}if(a.isBioAtom&&a.notCrossLinked&&d.getCrossLinkVector(null,!0,!0))break}else{if(-2147483648!=a.atomNumber&&a.atomNumber!=d.getAtomNumber())break;if(0<=a.jmolIndex&&d.getIndex()!=a.jmolIndex)break;if(null!=a.atomType&&!a.atomType.equals(d.getAtomType()))break;if(-2147483648!=(f=a.getAtomicMass())&&(0<=f&&f!=(e=d.getIsotopeNumber())||0>f&&0!=e&&-f!=e))break;if(!this.noAromatic&&!a.aromaticAmbiguous&&
a.isAromatic!=this.bsAromatic.get(b))break;if(-2147483648!=(f=a.getCharge())&&f!=d.getFormalCharge())break;f=a.getCovalentHydrogenCount()+a.explicitHydrogenCount;if(0<=f&&f!=d.getTotalHydrogenCount())break;if(-2147483648!=(f=a.implicitHydrogenCount))if(e=d.getImplicitHydrogenCount(),-1==f?0==e:f!=e)break;if(0<a.degree&&a.degree!=d.getCovalentBondCount()-d.getImplicitHydrogenCount())break;if(0<a.nonhydrogenDegree&&a.nonhydrogenDegree!=d.getCovalentBondCount()-d.getCovalentHydrogenCount())break;if(this.isSmarts&&
0<a.valence&&a.valence!=d.getTotalValence())break;if(0<a.connectivity&&a.connectivity!=d.getCovalentBondCountPlusMissingH())break;if(-2147483648!=a.atomNumber&&a.atomNumber!=d.getAtomNumber())break;if(0<=a.jmolIndex&&d.getIndex()!=a.jmolIndex)break;if(null!=a.atomType&&!a.atomType.equals(d.getAtomType()))break;if((!this.ignoreAtomClass||this.isSmarts)&&!Float.isNaN(a.atomClass)&&a.atomClass!=d.getFloatProperty("property_atomclass"))break;if(null!=this.ringData){if(-1<=a.ringSize)if(0>=a.ringSize){if(0==
this.ringCounts[b]!=(0==a.ringSize))break}else{d=this.ringData[500==a.ringSize?5:600==a.ringSize?6:a.ringSize];if(null==d||!d.get(b))break;if(!this.noAromatic)if(500==a.ringSize){if(!this.bsAromatic5.get(b))break}else if(600==a.ringSize&&!this.bsAromatic6.get(b))break}if(-1<=a.ringMembership&&(-1==a.ringMembership?0==this.ringCounts[b]:this.ringCounts[b]!=a.ringMembership))break;if(0<=a.ringConnectivity&&(f=this.ringConnections[b],-1==a.ringConnectivity&&0==f||-1!=a.ringConnectivity&&f!=a.ringConnectivity))break}}c=
!c;break}return c},"JS.SmilesAtom,~N");h(c$,"checkMatchBond",function(a,b,c,d,e,f){if(null!=c.bondsOr){for(var g=0;g<c.nBondsOr;g++)if(this.checkMatchBond(a,b,c.bondsOr[g],d,e,f))return!0;return!1}if(!this.isRingCheck&&!this.isTopology)if(0==c.nPrimitives){if(!this.checkPrimitiveBond(c,d,e,f))return!1}else for(a=0;a<c.nPrimitives;a++)if(b=c.setPrimitive(a),!this.checkPrimitiveBond(b,d,e,f))return!1;c.matchingBond=f;return!0},"JS.SmilesAtom,JS.SmilesAtom,JS.SmilesBond,~N,~N,JU.Edge");h(c$,"checkPrimitiveBond",
function(a,b,c,d){var e=!1;switch(a.order){case 96:return a.isNot!=(this.targetAtoms[c].getOffsetResidueAtom("\x00",1)==this.targetAtoms[b].getOffsetResidueAtom("\x00",0));case 112:return a.isNot!=this.targetAtoms[b].isCrossLinked(this.targetAtoms[c])}var f=!this.noAromatic&&this.bsAromatic.get(b),g=!this.noAromatic&&this.bsAromatic.get(c);d=d.getCovalentOrder();var j=a.order;if(f&&g)switch(j){case 17:case 65:e=JS.SmilesSearch.isRingBond(this.ringSets,null,b,c);break;case 1:e=!this.isSmarts||!JS.SmilesSearch.isRingBond(this.ringSets,
this.getBSAromaticRings(),b,c);break;case 2:e=this.isNormalized||this.aromaticDouble&&(2==d||514==d);break;case 65537:case 65538:e=!a.isNot;break;case 81:case -1:e=!0}else switch(j){case 17:if(!this.noAromatic)break;case 81:case -1:e=!0;break;case 1:case 1025:case 1041:switch(d){case 1:case 1025:case 1041:e=!0}break;case 65537:case 65538:switch(d){case 1:case 65537:case 65538:e=!a.isNot}break;case 2:case 3:case 4:e=d==j;break;case 65:e=JS.SmilesSearch.isRingBond(this.ringSets,null,b,c)}return e!=
a.isNot},"JS.SmilesBond,~N,~N,JU.Edge");h(c$,"getBSAromaticRings",function(){if(null==this.bsAromaticRings&&(this.bsAromaticRings=new JU.BS,null!=this.ringSets&&null!=this.bsAromatic))for(var a=this.ringSets.size();0<=--a;){var b=this.ringSets.get(a).clone();b.andNot(this.bsAromatic);b.isEmpty()&&this.bsAromaticRings.set(a)}return this.bsAromaticRings});c$.isRingBond=h(c$,"isRingBond",function(a,b,c,d){if(null!=a)for(var e=a.size();0<=--e;){var f=a.get(e);if(f.get(c)&&f.get(d)&&(null==b||b.get(e)))return!0}return!1},
"JU.Lst,JU.BS,~N,~N");h(c$,"checkStereochemistry",function(){for(var a=0;a<this.measures.size();a++)if(!this.measures.get(a).check())return!1;if(null!=this.stereo&&!this.stereo.checkStereoChemistry(this,this.v))return!1;if(!this.haveBondStereochemistry)return!0;for(var b=null,c=null,a=0;a<this.ac;a++){for(var d=this.patternAtoms[a],e=null,f=null,g=null,j=0,l=0,t=0,k=d.getBondCount(),m=!1,h=!0,p=0;p<k;p++){var c=d.getBond(p),q=c.atom2===d,h=c.atom1.index<c.atom2.index,r=c.order;switch(r){case 65537:case 65538:if(!h)continue;
case 2:if(q)continue;e=c.atom2;t=r;(m=2!=r)&&(j=c.isNot?-1:1);break;case 1025:case 1041:f=q?c.atom1:c.atom2,j=q!=(1025==r)?1:-1}}if(m){if(this.setAtropicity){null==b&&(b=new JU.Lst);b.addLast(c);continue}g=d.getBond(c.atropType[0]);if(null==g)return!1;f=g.getOtherAtom(d);g=e.getBond(c.atropType[1]);if(null==g)return!1;g=g.getOtherAtom(e);JU.Logger.debugging&&JU.Logger.info("atropisomer check for atoms "+f+d+" "+e+g)}else{if(null==e||0==j)continue;k=d;for(p=0;2==e.getBondCount()&&4==e.getValence();)p++,
q=e.getEdges(),q=q[q[0].getOtherAtomNode(e)===k?1:0],k=e,e=q.getOtherAtomNode(e);if(1==p%2)continue;k=e.getBondCount();for(p=0;p<k&&0==l;p++)switch(c=e.getBond(p),r=c.order,r){case 1025:case 1041:g=(q=c.atom2===e)?c.atom1:c.atom2,l=q!=(1025==r)?1:-1}if(0==l)continue}k=d.getMatchingAtom();p=e.getMatchingAtom();f=f.getMatchingAtom();g=g.getMatchingAtom();if(null==f||null==g)return!1;this.haveTopo&&this.setTopoCoordinates(k,p,f,g,t);f=JS.SmilesMeasure.setTorsionData(f,k,p,g,this.v,m);if(m){if(f*=j*(65537==
t?1:-1)*(h?1:-1),JU.Logger.debugging&&JU.Logger.info("atrop dihedral "+f+" "+d+" "+e+" "+c),1>f)return!1}else if(0>this.v.vTemp1.dot(this.v.vTemp2)*j*l)return!1}if(this.setAtropicity){this.atropKeys="";for(a=0;a<b.size();a++)this.atropKeys+=","+this.getAtropIndex(c=b.get(a),!0)+this.getAtropIndex(c,!1)}return!0});h(c$,"getAtropIndex",function(a,b){for(var c=b?a.atom1:a.atom2,d=c.getMatchingAtom(),d=JU.Edge.getAtropismNode(a.matchingBond.order,d,b),e=c.bonds,f=c.getBondCount();0<=--f;)if(e[f].getOtherAtomNode(c).getMatchingAtom()===
d)return f+1;return 0},"JS.SmilesBond,~B");h(c$,"setTopoCoordinates",function(a,b,c,d,e){a.set(-1,0,0);b.set(1,0,0);if(2!=e)e=a.getBondTo(b),b=65537==e.order?1:-1,c.set(-1,1,0),d.set(1,1,b/2);else{d=c=0;for(var f=a.getEdges(),g=f.length;0<=--g;)if(e=f[g],2!=e.order){var j=e.getOtherAtomNode(a);j.set(-1,0==c++?-1:1,0);j=e.getAtomIndex2()==a.getIndex()?c:-c;switch(e.order){case 1025:d=j;break;case 1041:d=-j}}var l=0;c=0;a=Array(2);f=b.getEdges();for(g=f.length;0<=--g;)if(e=f[g],2!=e.order)switch(j=
e.getOtherAtomNode(b),a[c]=j,j.set(1,0==c++?1:-1,0),j=e.getAtomIndex2()==b.getIndex()?c:-c,e.order){case 1025:l=j;break;case 1041:l=-j}0<d*l==(Math.abs(d)%2==Math.abs(l)%2)&&(b=a[0].y,a[0].y=a[1].y,a[1].y=b)}},"JS.SmilesAtom,JS.SmilesAtom,JS.SmilesAtom,JS.SmilesAtom,~N");h(c$,"createTopoMap",function(a){var b=null==a,c=this.getMissingHydrogenCount(),c=this.ac+c,d=Array(c);this.targetAtoms=d;for(var e=0,f=0;e<this.ac;e++,f++){var g=this.patternAtoms[e],j=g.explicitHydrogenCount;0>j&&(j=0);var l=d[f]=
(new JS.SmilesAtom).setTopoAtom(g.component,f,g.symbol,g.getCharge());l.implicitHydrogenCount=j;if(!b){l.mapIndex=e;l.stereo=g.stereo;l.setAtomicMass(g.getAtomicMass());l.bioAtomName=g.bioAtomName;l.residueName=g.residueName;l.residueChar=g.residueChar;l.residueNumber=g.residueNumber;l.atomNumber=g.residueNumber;l.insCode=g.insCode;l.atomClass=g.atomClass;l.explicitHydrogenCount=0;l.isBioAtom=g.isBioAtom;l.bioType=g.bioType;l.$isLeadAtom=g.$isLeadAtom;!b&&g.isAromatic&&a.set(f);g.setMatchingAtom(null,
f);g=Array(g.getBondCount()+j);for(l.setBonds(g);0<=--j;){g=d[++f]=(new JS.SmilesAtom).setTopoAtom(l.component,f,"H",0);g.mapIndex=-e-1;g.setBonds(Array(1));var t=new JS.SmilesBond(l,g,1,!1);JU.Logger.debugging&&JU.Logger.info(""+t)}}}if(!b){for(e=0;e<this.ac;e++){g=this.patternAtoms[e];j=g.getMatchingAtomIndex();a=d[j];j=g.getBondCount();for(b=0;b<j;b++)if(f=g.getBond(b),f.atom1===g){t=1;switch(f.order){case 1:case 2:case 3:case 4:case 1025:case 1041:case 65537:case 65538:case 112:case 96:t=f.order;
break;case 17:t=514}f=d[f.atom2.getMatchingAtomIndex()];t=new JS.SmilesBond(a,f,t,!1);f.bondCount--;JU.Logger.debugging&&JU.Logger.info(""+t)}else f=d[f.atom1.getMatchingAtomIndex()],t=f.getBondTo(a),a.addBond(t)}for(e=0;e<c;e++)if(t=d[e],g=t.bonds,!(2>g.length||g[0].isFromPreviousTo(t)))for(j=g.length;1<=--j;)if(g[j].isFromPreviousTo(t)){t=g[j];g[j]=g[0];g[0]=t;break}if(!this.ignoreStereochemistry)for(e=this.ac;0<=--e;)g=this.patternAtoms[e],null!=g.stereo&&g.stereo.fixStereo(g)}},"JU.BS");c$.normalizeAromaticity=
h(c$,"normalizeAromaticity",function(a,b,c){var d=new JS.SmilesSearch;d.setFlags(c);d.targetAtoms=a;d.targetAtomCount=a.length;d.bsSelected=JU.BSUtil.newBitSet2(0,a.length);c=JU.AU.createArrayOfArrayList(4);d.setRingData(null,c,!0);b.or(d.bsAromatic);if(!b.isEmpty()){b=c[3];for(d=b.size();0<=--d;){c=b.get(d);for(var e=c.nextSetBit(0);0<=e;e=c.nextSetBit(e+1)){var f=a[e];!f.isAromatic&&!(-2==f.elementNumber||0==f.elementNumber)&&f.setSymbol(f.symbol.toLowerCase())}}}},"~A,JU.BS,~N");h(c$,"getSelections",
function(){var a=this.top.htNested;if(!(null==a||0==this.targetAtoms.length))for(var b=new java.util.Hashtable,c,a=a.entrySet().iterator();a.hasNext()&&((c=a.next())||1);){var d=c.getValue().toString();if(d.startsWith("select")){var e=b.containsKey(d)?b.get(d):this.targetAtoms[0].findAtomsLike(d.substring(6));null==e&&(e=new JU.BS);b.put(d,e);c.setValue(e)}}});h(c$,"findImplicitHydrogen",function(a){for(var b=a.getEdges().length;0<=--b;){var c=a.getBondedAtomIndex(b);if(1==this.targetAtoms[c].getElementNumber()&&
!this.bsFound.get(c))return this.targetAtoms[c]}return null},"JU.Node");h(c$,"toString",function(){var a=(new JU.SB).append(this.pattern);a.append("\nmolecular formula: "+this.getMolecularFormula(!0,null,!1));return a.toString()});H(c$,"SUBMODE_NESTED",1,"SUBMODE_RINGCHECK",2,"SUBMODE_OR",3)});B("JS");C(["java.util.Hashtable","JU.BS","JS.VTemp"],"JS.SmilesGenerator","JU.AU $.Lst $.SB JS.InvalidSmilesException $.SmilesAtom $.SmilesBond $.SmilesParser $.SmilesSearch $.SmilesStereo JU.BSUtil $.Elements $.JmolMolecule $.Logger".split(" "),
function(){c$=D(function(){this.atoms=null;this.ac=0;this.bsAromatic=this.bsSelected=null;this.flags=0;this.explicitH=!1;this.vTemp=this.ringSets=null;this.nPairsMax=this.nPairs=0;this.bsIncludingH=this.bsRingKeys=this.htRings=this.htRingsSequence=this.prevSp2Atoms=this.prevAtom=this.bsToDo=this.bsBondsDn=this.bsBondsUp=null;this.topologyOnly=!1;this.getAromatic=!0;this.openSMILES=this.noStereo=this.aromaticDouble=this.noBioComment=this.addAtomComment=!1;this.smilesStereo=this.polySmilesCenter=null;
this.isPolyhedral=!1;this.sm=this.aromaticRings=null;this.iHypervalent=0;this.atemp=null;this.chainCheck=0;E(this,arguments)},JS,"SmilesGenerator");I(c$,function(){this.vTemp=new JS.VTemp;this.bsBondsUp=new JU.BS;this.bsBondsDn=new JU.BS;this.htRingsSequence=new java.util.Hashtable;this.htRings=new java.util.Hashtable;this.bsRingKeys=new JU.BS});h(c$,"getSmiles",function(a,b,c,d,e,f){var g=d.nextSetBit(0);if(0>g)return"";this.sm=a;this.flags=f;this.atoms=b;this.ac=c;this.bsSelected=d=JU.BSUtil.copy(d);
this.flags=f=JS.SmilesSearch.addFlags(f,null==e?"":e.toUpperCase());if(1048576==(f&1048576))return this.getBioSmiles(d,e,f);this.openSMILES=5==(f&5);this.addAtomComment=131072==(f&131072);this.aromaticDouble=512==(f&512);this.explicitH=4096==(f&4096);this.topologyOnly=8192==(f&8192);this.getAromatic=16!=(f&16);this.noStereo=32==(f&32);this.isPolyhedral=65536==(f&65536);return this.getSmilesComponent(b[g],d,!0,!1,!1)},"JS.SmilesMatcher,~A,~N,JU.BS,~S,~N");h(c$,"getBioSmiles",function(a,b,c){this.addAtomComment=
131072==(c&131072);var d=3145728==(c&3145728),e=34603008==(c&34603008),f=5242880==(c&5242880);c=9437184==(c&9437184);var g=f||c,j=new JU.SB;null!=b&&!this.noBioComment&&j.append("//* Jmol bioSMILES ").append(b.$replace("*","_")).append(" *//");b=this.noBioComment?"":"\n";var l=new JU.BS,t=null,k="",m,h=new JU.Lst;try{for(var p=0,q=a.nextSetBit(0);0<=q;q=a.nextSetBit(q+1)){var r=this.atoms[q],n=r.getGroup1("?"),y=r.getBioStructureTypeName(),v=n===n.toLowerCase();if(null!=b)if(0<j.length()&&j.append(b),
b=null,p=0,0<y.length)0!=r.getChainID()&&!e&&(m="//* chain "+r.getChainIDStr()+" "+y+" "+r.getResno()+" *// ",p=m.length,j.append(m)),p++,j.append("~").appendC(y.toLowerCase().charAt(0)).append("~");else{m=this.getSmilesComponent(r,a,!1,!0,!0);if(m.equals(t)){b="";continue}var t=m,x=r.getGroup3(!0),w;e?w="/"+m+"/":(null!=x&&(m="//* "+x+" *//"+m),w=m+"//");if(0<=k.indexOf(w)){b="";continue}k+=w;j.append(m);b=e?".":".\n";continue}75<=p&&!e&&(j.append("\n  "),p=2);this.addAtomComment&&j.append("\n//* ["+
r.getGroup3(!1)+"#"+r.getResno()+"] *//\t");v?this.addBracketedBioName(j,r,0<y.length?".0":null,!1):j.append(n);p++;if(g){r.getCrossLinkVector(h,f,c);for(var z=0;z<h.size();z+=3)j.append(":"),m=this.getRingCache(h.get(z).intValue(),h.get(z+1).intValue(),this.htRingsSequence),j.append(m),p+=1+m.length;h.clear()}r.getGroupBits(l);a.andNot(l);var A=r.getOffsetResidueAtom("\x00",1);if(0>A||!a.get(A)){e||j.append(" //* ").appendI(r.getResno()).append(" *//");if(0>A&&0>(A=a.nextSetBit(q+1)))break;0<p&&
(b=e?".":".\n")}q=A-1}}catch(s){if(G(s,Exception))throw new JS.InvalidSmilesException("//* error: "+s.getMessage()+" *//");throw s;}if(!d&&!this.htRingsSequence.isEmpty())throw this.dumpRingKeys(j,this.htRingsSequence),new JS.InvalidSmilesException("//* ?ring error? *//");m=j.toString();m.endsWith(".\n")?m=m.substring(0,m.length-2):e&&m.endsWith(".")&&(m=m.substring(0,m.length-1));return m},"JU.BS,~S,~N");h(c$,"addBracketedBioName",function(a,b,c,d){a.append("[");if(null!=c){var e=b.getChainIDStr();
a.append(b.getGroup3(!1));c.equals(".0")||a.append(c).append("#").appendI(b.getElementNumber());d&&(a.append("//* ").appendI(b.getResno()),0<e.length&&a.append(":").append(e),a.append(" *//"))}else a.append(JU.Elements.elementNameFromNumber(b.getElementNumber()));a.append("]")},"JU.SB,JU.Node,~S,~B");h(c$,"getSmilesComponent",function(a,b,c,d,e){!this.explicitH&&(1==a.getElementNumber()&&0<a.getEdges().length)&&(a=this.atoms[a.getBondedAtomIndex(0)]);this.bsSelected=JU.JmolMolecule.getBranchBitSet(this.atoms,
a.getIndex(),JU.BSUtil.copy(b),null,-1,!0,c);b.andNot(this.bsSelected);this.iHypervalent=-1;for(b=this.bsSelected.nextSetBit(0);0<=b&&0>this.iHypervalent;b=this.bsSelected.nextSetBit(b+1))if(4<this.atoms[b].getCovalentBondCount()||this.isPolyhedral)this.iHypervalent=b;this.bsIncludingH=JU.BSUtil.copy(this.bsSelected);if(!this.explicitH)for(b=this.bsSelected.nextSetBit(0);0<=b;b=this.bsSelected.nextSetBit(b+1))c=this.atoms[b],1==c.getElementNumber()&&(0==c.getIsotopeNumber()&&0<c.getBondCount()&&c.getBondedAtomIndex(0)!=
this.iHypervalent)&&this.bsSelected.clear(b);this.bsAromatic=new JU.BS;!this.topologyOnly&&2<this.bsSelected.cardinality()&&(this.generateRingData(),this.setBondDirections());this.bsToDo=JU.BSUtil.copy(this.bsSelected);c=new JU.SB;for(b=this.bsToDo.nextSetBit(0);0<=b;b=this.bsToDo.nextSetBit(b+1))if(4<this.atoms[b].getCovalentBondCount()||this.isPolyhedral)null==a&&c.append("."),this.getSmilesAt(c,this.atoms[b],d,!1,e),a=null;if(null!=a)for(;null!=(a=this.getSmilesAt(c,a,d,!0,e)););for(;0<this.bsToDo.cardinality()||
!this.htRings.isEmpty();){a=this.htRings.values().iterator();if(a.hasNext()){if(a=this.atoms[a.next()[1].intValue()],!this.bsToDo.get(a.getIndex()))break}else a=this.atoms[this.bsToDo.nextSetBit(0)];c.append(".");for(this.prevAtom=this.prevSp2Atoms=null;null!=(a=this.getSmilesAt(c,a,d,!0,e)););}if(!this.htRings.isEmpty())throw this.dumpRingKeys(c,this.htRings),new JS.InvalidSmilesException("//* ?ring error? *//\n"+c);d=c.toString();if(0<=d.indexOf("^-")){e=d;try{var f=this.sm.getAtropisomerKeys(d,
this.atoms,this.ac,this.bsSelected,this.bsAromatic,this.flags);for(b=1;b<f.length;){var g=d.indexOf("^-");if(0>g)break;d=d.substring(0,g+1)+f.substring(b,b+2)+d.substring(g+1);b+=3}}catch(j){if(G(j,Exception))System.out.println("???"),d=e;else throw j;}}return d},"JU.Node,JU.BS,~B,~B,~B");h(c$,"generateRingData",function(){var a=JS.SmilesParser.newSearch("[r500]",!0,!0);a.targetAtoms=this.atoms;a.setSelected(this.bsSelected);a.setFlags(this.flags);a.targetAtomCount=this.ac;a.ringDataMax=7;a.flags=
this.flags;var b=JU.AU.createArrayOfArrayList(4);a.setRingData(null,b,!0);this.bsAromatic=a.bsAromatic;this.ringSets=a.ringSets;this.aromaticRings=b[3]});h(c$,"getBondStereochemistry",function(a,b){if(null==a)return"\x00";var c=a.index,d=null==b||a.getAtomIndex1()==b.getIndex();return this.bsBondsUp.get(c)?d?"/":"\\":this.bsBondsDn.get(c)?d?"\\":"/":"\x00"},"JU.Edge,JU.Node");h(c$,"setBondDirections",function(){for(var a=new JU.BS,b=K(2,3,null),c=this.bsSelected.nextSetBit(0);0<=c;c=this.bsSelected.nextSetBit(c+
1))for(var d=this.atoms[c],e=d.getEdges(),f=0;f<e.length;f++){var g=e[f],j=g.index,l;if(!a.get(j)&&!(2!=g.getCovalentOrder()||JS.SmilesSearch.isRingBond(this.ringSets,null,c,(l=g.getOtherAtomNode(d)).getIndex()))){a.set(j);g=0;for(j=d;2==l.getCovalentBondCount()&&4==l.getValence();){var h=l.getEdges(),h=h[h[0].getOtherAtomNode(l)===j?1:0];a.set(h.index);j=l;l=h.getOtherAtomNode(l);g++}if(1!=g%2){for(var k=g=null,j=0,h=K(-1,[d,l]),k=1,m=0;2>m&&0<k&&3>k;m++)for(var k=0,n=h[m],p=n.getEdges(),q=0;q<p.length;q++)1!=
p[q].getCovalentOrder()||1==p[q].getOtherAtomNode(n).getElementNumber()||(b[m][k++]=p[q],"\x00"!=this.getBondStereochemistry(p[q],n)&&(g=p[q],j=m));if(!(3==k||0==k))if(null==g&&(j=0,g=b[j][0],this.bsBondsUp.set(g.index)),n=this.getBondStereochemistry(g,h[j]),k=g.getOtherAtomNode(h[j]),null!=k)for(m=0;2>m;m++)for(p=0;2>p;p++)if(q=b[m][p],!(null==q||q===g)){var r=q.index,u=q.getOtherAtomNode(h[m]);if(null!=u){var y=this.getBondStereochemistry(q,h[m]),v=JS.SmilesStereo.isDiaxial(h[j],h[m],k,u,this.vTemp,
0);"\x00"==y||y!=n==v?("\\"==n&&v||"/"==n&&!v)==(q.getAtomIndex1()!=u.getIndex())?this.bsBondsUp.set(r):this.bsBondsDn.set(r):JU.Logger.error("BOND STEREOCHEMISTRY ERROR");JU.Logger.debugging&&JU.Logger.debug(this.getBondStereochemistry(g,h[0])+" "+k.getIndex()+" "+u.getIndex()+" "+this.getBondStereochemistry(q,h[m]))}}}}}});h(c$,"getSmilesAt",function(a,b,c,d,e){var f=b.getIndex();if(!this.bsToDo.get(f))return null;this.bsToDo.clear(f);var g=f==this.iHypervalent||this.explicitH,j=!this.bsSelected.get(f),
l=null==this.prevAtom?-1:this.prevAtom.getIndex(),h=this.bsAromatic.get(f),k=null!=this.prevSp2Atoms,m=b.getElementNumber(),n=0,p=new JU.Lst,q=null,r=null,u=b.getEdges();null!=this.polySmilesCenter&&(d=!1,this.sortBonds(b,this.prevAtom,this.polySmilesCenter));var y=null,v=h?10:0;JU.Logger.debugging&&JU.Logger.debug(a.toString());if(null!=u)for(var x=u.length;0<=--x;){var w=u[x];if(w.isCovalent()){var z=u[x].getOtherAtomNode(b),A=z.getIndex();if(A==l)r=u[x];else{var s=!g&&1==z.getElementNumber()&&
0==z.getIsotopeNumber();if(!this.bsIncludingH.get(A))if(!s&&c&&this.bsSelected.get(f))this.bsToDo.set(A);else continue;s?(y=z,n++,1<n&&(v=10)):p.addLast(u[x])}}}z=this.prevSp2Atoms;u=null==z?1:2;null==z&&(z=Array(5));var B=null;null!=r&&(B=this.getBondOrder(r,f,l,h),k||(z[0]=this.prevAtom));var u=u+n,A=0,C=new JU.BS,E=p.size();if(d)for(x=0;x<E;x++){var w=p.get(x),s=w.getOtherAtomNode(b),l=s.getCovalentBondCount()-(g?0:s.getCovalentHydrogenCount()),D=w.getCovalentOrder();if(1==l&&(null!=q||x<E-1))C.set(w.index);
else if((1<D||l>A)&&!this.htRings.containsKey(JS.SmilesGenerator.getRingKey(s.getIndex(),f)))A=1<D?1E3+D:l,q=w}g=null==q?null:q.getOtherAtomNode(b);l=null==q?0:q.getCovalentOrder();if(h||2==l&&1<n||null!=g&&JS.SmilesSearch.isRingBond(this.ringSets,null,f,g.getIndex()))z=null;A=Array(7);7>v&&null!=r&&(k&&2==r.getCovalentOrder()&&2==l&&null!=this.prevSp2Atoms[1]?(A[v++]=this.prevSp2Atoms[0],A[v++]=this.prevSp2Atoms[1]):A[v++]=this.prevAtom);7>v&&1==n&&(A[v++]=y);E=1==l&&null==this.prevSp2Atoms;w=this.getBondStereochemistry(r,
this.prevAtom);if(null!=B||"\x00"!=w)"\x00"!=w&&(B=""+w),a.append(B);for(var D=v,G=u,y=new JU.SB,r=new JU.Lst,x=0;x<p.size();x++)w=p.get(x),C.get(w.index)&&(s=w.getOtherAtomNode(b),w=new JU.SB,this.prevAtom=b,this.prevSp2Atoms=null,this.getSmilesAt(w,s,c,d,e),w=w.toString(),p.removeItemAt(x--),null==q?r.addLast(w):y.append("(").append(w).append(")"),7>v&&(A[v++]=s),null!=z&&5>u&&(z[u++]=s));c=new JU.SB;var F=v,H=u,C=null;if(!d&&!this.noStereo&&null==this.polySmilesCenter&&(5==p.size()||6==p.size()))C=
this.sortInorganic(b,p,this.vTemp);for(x=0;x<p.size();x++)w=p.get(x),w!==q&&(s=w.getOtherAtomNode(b),B=this.getBondOrder(w,f,s.getIndex(),h),E||(w=this.getBondStereochemistry(w,b),"\x00"!=w&&(B=""+w)),c.append(B),c.append(this.getRingCache(f,s.getIndex(),this.htRings)),7>v&&(A[v++]=s),null!=z&&5>u&&(z[u++]=s));D!=F&&F!=v&&this.swapArray(A,D,F,v);G!=H&&H!=u&&this.swapArray(z,G,H,u);if(k&&2==v&&2==l&&3==g.getCovalentBondCount()){u=g.getEdges();for(d=0;d<u.length;d++)u[d].isCovalent()&&g.getBondedAtomIndex(d)!=
f&&(A[v++]=this.atoms[g.getBondedAtomIndex(d)]);u=0}else null!=g&&7>v&&(A[v++]=g);d=b.getFormalCharge();k=b.getIsotopeNumber();p=b.getValence();x=this.openSMILES?b.getFloatProperty("property_atomclass"):NaN;s=b.getAtomName();w=b.getBioStructureTypeName();this.addAtomComment&&a.append("\n//* "+b.toString()+" *//\t");this.topologyOnly?a.append("*"):j&&0!=w.length&&0!=s.length?this.addBracketedBioName(a,b,"."+s,!1):a.append(JS.SmilesAtom.getAtomLabel(m,k,e?-1:p,d,x,n,h,null!=C?C:this.noStereo?null:this.checkStereoPairs(b,
f,A,v)));a.appendSB(c);if(null!=q)a.appendSB(y);else{l=r.size()-1;if(0<=l){for(x=0;x<l;x++)a.append("(").append(r.get(x)).append(")");a.append(r.get(l))}return null}null!=z&&2==l&&(1==u||2==u)?(null==z[0]&&(z[0]=b),null==z[1]&&(z[1]=b)):z=null;this.prevSp2Atoms=z;this.prevAtom=b;return g},"JU.SB,JU.Node,~B,~B,~B");h(c$,"swapArray",function(a,b,c,d){b=c-b;if(null==this.atemp||this.atemp.length<b)this.atemp=Array(b);for(var e=b,f=c;0<e;)this.atemp[--e]=a[--f];for(f=c;f<d;f++)a[f-b]=a[f];e=b;for(f=d;0<
e;)a[--f]=this.atemp[--e]},"~A,~N,~N,~N");h(c$,"getBondOrder",function(a,b,c,d){if(this.topologyOnly)return"";if(65537==(a.order&65537))return"^-";a=a.getCovalentOrder();return!d||!this.bsAromatic.get(c)?JS.SmilesBond.getBondOrderString(a):1==a&&!this.isSameAromaticRing(b,c)?"-":this.aromaticDouble&&(2==a||514==a)?"=":""},"JU.Edge,~N,~N,~B");h(c$,"isSameAromaticRing",function(a,b){for(var c,d=this.aromaticRings.size();0<=--d;)if((c=this.aromaticRings.get(d)).get(a)&&c.get(b))return!0;return!1},"~N,~N");
h(c$,"sortBonds",function(a,b,c){if(null==this.smilesStereo)try{this.smilesStereo=JS.SmilesStereo.newStereo(null)}catch(d){if(!G(d,JS.InvalidSmilesException))throw d;}this.smilesStereo.sortBondsByStereo(a,b,c,a.getEdges(),this.vTemp.vA)},"JU.Node,JU.Node,JU.P3");h(c$,"sortInorganic",function(a,b,c){for(var d=a.getIndex(),e=b.size(),f=new JU.Lst,g=new JU.Lst,j,l,h=null,k=null,m,n,p=new JU.BS,q=null,r=Array(6),q=!0,u="",y=0,v=0;v<e;v++)if(m=b.get(v),r[0]=j=m.getOtherAtomNode(a),0==v?u=this.addStereoCheck(0,
d,j,"",null):q&&null!=this.addStereoCheck(0,d,j,u,null)&&(q=!1),!p.get(v)){p.set(v);for(var x=!1,w=v+1;w<e;w++)if(!p.get(w)&&(n=b.get(w),l=n.getOtherAtomNode(a),JS.SmilesStereo.isDiaxial(a,a,j,l,c,-0.95))){switch(++y){case 1:h=j;break;case 2:k=j;break;case 3:2==JS.SmilesStereo.getHandedness(k,h,j,a,c)&&(j=m,m=n,n=j)}f.addLast(K(-1,[m,n]));x=!0;p.set(w);break}x||g.addLast(m)}d=f.size();if(q||6==e&&3!=d||5==e&&0==d)return"";q=f.get(0);m=q[0];r[0]=m.getOtherAtomNode(a);b.clear();b.addLast(m);1<d&&g.addLast(f.get(1)[0]);
3==d&&g.addLast(f.get(2)[0]);1<d&&g.addLast(f.get(1)[1]);3==d&&g.addLast(f.get(2)[1]);for(v=0;v<g.size();v++)m=g.get(v),b.addLast(m),r[v+1]=m.getOtherAtomNode(a);b.addLast(q[1]);r[e-1]=q[1].getOtherAtomNode(a);return JS.SmilesStereo.getStereoFlag(a,r,e,c)},"JU.Node,JU.Lst,JS.VTemp");h(c$,"checkStereoPairs",function(a,b,c,d){if(4>d)return"";if(4==d&&6==a.getElementNumber())for(var e="",f=0;4>f;f++)if(null==(e=this.addStereoCheck(0,b,c[f],e,JU.BSUtil.newAndSetBit(b)))){d=10;break}return 6<d?"":JS.SmilesStereo.getStereoFlag(a,
c,d,this.vTemp)},"JU.Node,~N,~A,~N");h(c$,"addStereoCheck",function(a,b,c,d,e){null!=e&&e.set(b);var f=c.getAtomicAndIsotopeNumber(),g=c.getCovalentBondCount(),j=6==f&&!this.explicitH?c.getCovalentHydrogenCount():0;if(6==f?4!=g:1==f||1<g)return d;var l=";"+a+"/"+f+"/"+j+"/"+g+(0==a?",":"_");if(6==f)switch(j){case 1:return d+l+ ++this.chainCheck;case 0:case 2:if(null==e)return d;for(var h=c.getEdges(),k="",m="",n=2==j?0:3,f=c.getBondCount();0<=--f;){var m=h[f].getOtherAtomNode(c),p=m.getIndex();!e.get(p)&&
(h[f].isCovalent()&&1!=m.getElementNumber())&&(e.set(p),m=this.addStereoCheck(a+1,c.getIndex(),m,"",e),0<=k.indexOf(m)&&n--,k+=m)}if(3==n)return d+l+ ++this.chainCheck;l=(l+k).$replace(",","_");if(0<a)return d+l}if(0<=d.indexOf(l)){if(3==j){for(f=a=0;f<g&&3>a;f++)e=c.getBondedAtomIndex(f),e!=b&&(a+=this.atoms[e].getAtomicAndIsotopeNumber());if(3<a)return d}return null}return d+l},"~N,~N,JU.Node,~S,JU.BS");h(c$,"getRingCache",function(a,b,c){var d=JS.SmilesGenerator.getRingKey(a,b),e=c.get(d),f=null==
e?null:e[0];if(null==f)this.bsRingKeys.set(++this.nPairs),this.nPairsMax=Math.max(this.nPairs,this.nPairsMax),c.put(d,K(-1,[f=this.getRingPointer(this.nPairs),Integer.$valueOf(b),Integer.$valueOf(this.nPairs)])),JU.Logger.debugging&&JU.Logger.debug("adding for "+a+" ring key "+this.nPairs+": "+d);else{c.remove(d);a=e[2].intValue();this.bsRingKeys.clear(a);if(0>this.bsRingKeys.nextSetBit(0)&&(2==this.nPairsMax||99==this.nPairsMax))this.nPairsMax=this.nPairs=99==this.nPairsMax?10:0;JU.Logger.debugging&&
JU.Logger.debug("using ring key "+d)}return f},"~N,~N,java.util.Map");h(c$,"getRingPointer",function(a){return 10>a?""+a:100>a?"%"+a:"%("+a+")"},"~N");h(c$,"dumpRingKeys",function(a,b){JU.Logger.info(a.toString()+"\n\n");for(var c,d=b.keySet().iterator();d.hasNext()&&((c=d.next())||1);)JU.Logger.info("unmatched connection: "+c)},"JU.SB,java.util.Map");c$.getRingKey=h(c$,"getRingKey",function(a,b){return Math.min(a,b)+"_"+Math.max(a,b)},"~N,~N")});B("JS");C(null,"JS.SmilesAromatic","java.util.Hashtable JU.BS $.Lst $.Measure $.V3 JS.SmilesRing $.SmilesRingSet JU.BSUtil $.Logger".split(" "),
function(){c$=O(JS,"SmilesAromatic");c$.setAromatic=h(c$,"setAromatic",function(a,b,c,d,e,f,g,j,l,h,k,m,n,p){g=g||0<f;if(p)for(w=d.size();0<=--w;){if(s=d.get(w),p=JS.SmilesAromatic.isSp2Ring(a,b,c,s,j?3.4028235E38:0<f?0.1:0.01,l,0==f)){e.or(s);if(g){h=new JU.Lst;for(var q=s.nextSetBit(0);0<=q;q=s.nextSetBit(q+1))for(var r=b[q],u=r.getEdges(),y=r.getIndex(),v=u.length;0<=--v;){var x=u[v].getOtherAtomNode(r).getIndex();x>y&&s.get(x)&&h.addLast(u[v])}switch(JS.SmilesAromatic.checkHueckelAromatic(a,b,
e,s,f,n)){case -1:continue;case 0:p=!1;case 1:if(null!=m&&m.addLast(new JS.SmilesRing(a,s,h,p)),!p)continue}}k.addLast(s)}}else for(var w=d.size();0<=--w;){var s=JU.BSUtil.copy(d.get(w));s.and(e);s.cardinality()==a&&k.addLast(s)}},"~N,~A,JU.BS,JU.Lst,JU.BS,~N,~B,~B,~B,JS.VTemp,JU.Lst,JU.Lst,~A,~B");c$.checkAromaticDefined=h(c$,"checkAromaticDefined",function(a,b,c){for(var d=b.nextSetBit(0);0<=d;d=b.nextSetBit(d+1))for(var e=a[d].getEdges(),f=0;f<e.length;f++)switch(e[f].order){case 515:case 514:case 513:c.set(e[f].getAtomIndex1()),
c.set(e[f].getAtomIndex2())}},"~A,JU.BS,JU.BS");c$.isSp2Ring=h(c$,"isSp2Ring",function(a,b,c,d,e,f,g){if(f)for(var j=d.nextSetBit(0);0<=j;j=d.nextSetBit(j+1)){if(3<b[j].getCovalentBondCount())return!1}else for(j=d.nextSetBit(0);0<=j;j=d.nextSetBit(j+1))if(3<b[j].getCovalentBondCountPlusMissingH())return!1;if(3.4028235E38==e)return!0;0>=e&&(e=0.01);for(var l=f=a=null,j=d.cardinality(),h=Array(2*j),k=0,m=1-5*e,j=d.nextSetBit(0);0<=j;j=d.nextSetBit(j+1)){for(var n=b[j],p=-1,q=-1,r=-1,u=n.getEdges().length;0<=
--u;){var s=n.getBondedAtomIndex(u);if(c.get(s))if(d.get(s))0>q?q=s:r=s;else{if(16==n.getElementNumber()){if(!g)return!1;s=-1}p=s}}null==l&&(l=new JU.V3,a=new JU.V3,f=new JU.V3);u=0;for(n=j;2>u;u++){JU.Measure.getNormalThroughPoints(b[q],b[n],b[r],a,f);if(!JS.SmilesAromatic.addNormal(a,l,m))return!1;h[k++]=JU.V3.newV(a);if(0>(n=p))break}}return JS.SmilesAromatic.checkStandardDeviation(h,l,k,e)},"~N,~A,JU.BS,JU.BS,~N,~B,~B");c$.addNormal=h(c$,"addNormal",function(a,b,c){var d=b.dot(a);if(0!=d&&Math.abs(d)<
c)return!1;0>d&&a.scale(-1);b.add(a);b.normalize();return!0},"JU.V3,JU.V3,~N");c$.checkStandardDeviation=h(c$,"checkStandardDeviation",function(a,b,c,d){for(var e=0,f=0,g=0;g<c;g++)var j=a[g].dot(b),e=e+j,f=f+j*j;e=Math.sqrt((f-e*e/c)/(c-1));return e<d},"~A,JU.V3,~N,~N");c$.checkHueckelAromatic=h(c$,"checkHueckelAromatic",function(a,b,c,d,e,f){for(var g=0,j=0,l=d.nextSetBit(0);0<=l&&0<=g;l=d.nextSetBit(l+1)){var h=b[l],k=h.getElementNumber(),m=h.getCovalentBondCountPlusMissingH(),m=m+h.getValence(),
m=m-4;if(6==k){var n=h.getFormalCharge();-2147483648!=n&&(m+=n)}k=5<=k&&8>=k?k-5:15==k?2:34==k?3:33==k?4:16==k?5:-1;if(0<=k){k=JS.SmilesAromatic.OS_PI_COUNTS[k];if(0>m||m>=k.length)return-1;switch(m=k[m]){case -2:return-1;case -1:k=h.getEdges();m=0;for(n=k.length;0<=--n;){var p=k[n];if(2==p.getCovalentOrder()){m=p.getOtherAtomNode(h);m=6==m.getElementNumber()||c.get(m.getIndex())?1:0<e?-100:0;break}}default:if(0>m)return-1;null!=f&&(f[l]=m);g+=m;1==m&&j++;JU.Logger.debugging&&JU.Logger.info("atom "+
h+" pi="+m+" npi="+g)}}}return 0==(g-2)%4&&(2>e||5==a||6==j)?1:0},"~N,~A,JU.BS,JU.BS,~N,~A");c$.finalizeAromatic=h(c$,"finalizeAromatic",function(a,b,c,d,e,f,g){g&&JS.SmilesAromatic.removeBridgingRings(c,d);JS.SmilesAromatic.checkFusedRings(d,e,c);b.clearAll();for(e=c.size();0<=--e;)b.or(c.get(e));if(g||f)for(e=b.nextSetBit(0);0<=e;e=b.nextSetBit(e+1)){c=a[e].getEdges();f=0;for(var j=c.length;0<=--j;){var l=c[j].getOtherAtomNode(a[e]),h=c[j].getCovalentOrder(),k=l.getIndex();if(b.get(k)){if(2==h){l=
!1;for(h=d.size();0<=--h;){var m=d.get(h);if(m.get(e)&&m.get(k)){l=!0;break}}if(!l){f=-1;break}}f++}else if(g&&6==l.getElementNumber()&&2==h){f=-1;break}}2>f&&(b.clear(e),e=-1)}},"~A,JU.BS,JU.Lst,JU.Lst,~A,~B,~B");c$.removeBridgingRings=h(c$,"removeBridgingRings",function(a,b){var c=new JU.BS,d=new JU.BS,e=new JU.BS;JS.SmilesAromatic.checkBridges(a,d,a,d,c);JS.SmilesAromatic.checkBridges(b,e,b,e,c);JS.SmilesAromatic.checkBridges(a,d,b,e,c);for(c=a.size();0<=--c;)d.get(c)&&a.removeItemAt(c);for(c=
b.size();0<=--c;)e.get(c)&&b.removeItemAt(c)},"JU.Lst,JU.Lst");c$.checkBridges=h(c$,"checkBridges",function(a,b,c,d,e){for(var f=a===c,g=a.size();0<=--g;)for(var j=a.get(g),l=f?g+1:0,h=c.size();--h>=l;){var k=c.get(h);k.equals(j)||(e.clearAll(),e.or(j),e.and(k),2<e.cardinality()&&(b.set(g),d.set(h)))}},"JU.Lst,JU.BS,JU.Lst,JU.BS,JU.BS");c$.checkFusedRings=h(c$,"checkFusedRings",function(a,b,c){for(var d=new java.util.Hashtable,e=a.size();0<=--e;){for(var f=a.get(e),g=f.edges,j=g.size();0<=--j;){var l=
JS.SmilesRing.getSetByEdge(g.get(j),d);null==l||l===f.$set||(null!=f.$set?l.addSet(f.$set,d):l.addRing(f))}(null==f.$set?f.$set=new JS.SmilesRingSet:f.$set).addRing(f);f.addEdges(d)}for(e=a.size();0<=--e;)if(!(f=a.get(e)).isOK&&!(null==(l=f.$set)||l.isEmpty())){if(2==l.getElectronCount(b)%4)for(j=l.size();0<=--j;)(f=l.get(j)).isOK||c.addLast(f);l.clear()}},"JU.Lst,~A,JU.Lst");H(c$,"OS_PI_COUNTS",K(-1,[s(-1,[-2,1,0]),s(-1,[1,2,1,-1]),s(-1,[2,1,2,1,1]),s(-1,[2,1]),s(-1,[-2,1,2,1,-2]),s(-1,[2,1,2,2])]))});
B("JS");C(["JU.BS"],"JS.SmilesRing",null,function(){c$=D(function(){this.bsEdgesToCheck=this.edges=this.$set=null;this.isOK=!1;this.n=0;E(this,arguments)},JS,"SmilesRing",JU.BS);F(c$,function(a,b,c,d){M(this,JS.SmilesRing,[]);this.or(b);this.edges=c;this.isOK=d;this.n=a},"~N,JU.BS,JU.Lst,~B");h(c$,"addEdges",function(a){for(var b=this.edges.size();0<=--b;)a.put(JS.SmilesRing.getKey(this.edges.get(b)),this.$set)},"java.util.Hashtable");c$.getSetByEdge=h(c$,"getSetByEdge",function(a,b){return b.get(JS.SmilesRing.getKey(a))},
"JU.Edge,java.util.Hashtable");c$.getKey=h(c$,"getKey",function(a){var b=a.getAtomIndex1();a=a.getAtomIndex2();return b<a?b+"_"+a:a+"_"+b},"JU.Edge")});B("JS");C(["JU.Lst","$.BS"],"JS.SmilesRingSet",null,function(){c$=D(function(){this.bs=null;E(this,arguments)},JS,"SmilesRingSet",JU.Lst);I(c$,function(){this.bs=new JU.BS});F(c$,function(){M(this,JS.SmilesRingSet,[])});h(c$,"addSet",function(a,b){for(var c=a.size();0<=--c;){var d=a.get(c);this.addRing(d);d.addEdges(b)}},"JS.SmilesRingSet,java.util.Hashtable");
h(c$,"addRing",function(a){this.addLast(a);a.$set=this;this.bs.or(a)},"JS.SmilesRing");h(c$,"getElectronCount",function(a){for(var b=0,c=this.bs.nextSetBit(0);0<=c;c=this.bs.nextSetBit(c+1))b+=a[c];return b},"~A")});B("JS");C(["JU.P3","JU.Node"],"JS.SmilesAtom",["java.lang.Float","JU.AU","JU.Elements","$.Logger"],function(){c$=D(function(){this.pattern=null;this.primitiveType=0;this.isAND=!1;this.subAtoms=null;this.index=this.nSubAtoms=0;this.residueChar=this.residueName=this.referance=null;this.insCode=
"\x00";this.isBioAtomWild=this.isBioResidue=this.isBioAtom=!1;this.bioType="\x00";this.$isLeadAtom=!1;this.notBondedIndex=-1;this.notCrossLinked=!1;this.aromaticAmbiguous=!0;this.covalentHydrogenCount=-1;this.elementDefined=this.hasSymbol=this.selected=this.not=!1;this.bioAtomName=this.atomType=null;this.isFirst=!0;this.jmolIndex=-1;this.elementNumber=-2;this.implicitHydrogenCount=this.explicitHydrogenCount=this.residueNumber=this.atomNumber=-2147483648;this.bonds=this.parent=null;this.iNested=this.bondCount=
0;this.isAromatic=!1;this.charge=this.atomicMass=-2147483648;this.matchingIndex=-1;this.stereo=null;this.component=-2147483648;this.atomSite=this.matchingComponent=0;this.nonhydrogenDegree=this.degree=-1;this.valence=0;this.connectivity=-1;this.ringSize=this.ringMembership=-2147483648;this.ringConnectivity=-1;this.matchingNode=null;this.hasSubpattern=!1;this.mapIndex=-1;this.atomClass=NaN;this.symbol=null;this.isTopoAtom=!1;this.missingHydrogenCount=0;E(this,arguments)},JS,"SmilesAtom",JU.P3,JU.Node);
I(c$,function(){this.bonds=Array(4)});c$.allowSmilesUnbracketed=h(c$,"allowSmilesUnbracketed",function(a){return 0<="B, C, N, O, P, S, F, Cl, Br, I, *,".indexOf(a+",")},"~S");F(c$,function(){M(this,JS.SmilesAtom,[])});n(c$,"getAtomType",function(){return null==this.atomType?this.bioAtomName:this.atomType});h(c$,"getChiralClass",function(){return null==this.stereo?-2147483648:this.stereo.getChiralClass(this)});h(c$,"isDefined",function(){return this.hasSubpattern||0!=this.iNested||this.isBioAtom||
-2147483648!=this.component||-2!=this.elementNumber||0<this.nSubAtoms});h(c$,"setBioAtom",function(a){this.isBioAtom="\x00"!=a;this.bioType=a;null!=this.parent&&(this.parent.bioType=a,this.parent.isBioAtom=this.isBioAtom,this.parent.isBioAtomWild=this.isBioAtomWild)},"~S");h(c$,"setAtomName",function(a){null!=a&&(0<a.length&&(this.bioAtomName=a),a.equals("\x00")&&(this.$isLeadAtom=!0),null!=this.parent&&(this.parent.bioAtomName=a))},"~S");h(c$,"setBonds",function(a){this.bonds=a},"~A");h(c$,"addSubAtom",
function(a,b){this.isAND=b;null==this.subAtoms&&(this.subAtoms=Array(2));this.nSubAtoms>=this.subAtoms.length&&(this.subAtoms=JU.AU.doubleLength(this.subAtoms));a.setIndex(this.index);a.parent=this;this.subAtoms[this.nSubAtoms++]=a;this.setSymbol("*");this.hasSymbol=!1;return a},"JS.SmilesAtom,~B");h(c$,"setIndex",function(a){this.index=a;return this},"~N");h(c$,"setTopoAtom",function(a,b,c,d){this.component=a;this.index=b;this.setSymbol(c);this.charge=d;this.isTopoAtom=!0;return this},"~N,~N,~S,~N");
h(c$,"setHydrogenCount",function(){this.missingHydrogenCount=this.explicitHydrogenCount;if(-2147483648!=this.explicitHydrogenCount)return!0;var a=JS.SmilesAtom.getDefaultCount(this.elementNumber,this.isAromatic);if(0>a)return this.missingHydrogenCount=0,-1==a;7==this.elementNumber&&this.isAromatic&&2==this.bondCount&&1==this.bonds[0].order&&1==this.bonds[1].order&&a++;for(var b=0;b<this.bondCount;b++){var c=this.bonds[b];switch(c.order){case 81:7==this.elementNumber&&JU.Logger.info("Ambiguous bonding to aromatic N found -- MF may be in error");
a-=1;break;case 1025:case 1041:case 65537:case 65538:a-=1;break;case 2:a-=this.isAromatic&&6==this.elementNumber?1:2;break;case 1:case 3:case 4:a-=c.order}}0<=a&&(this.missingHydrogenCount=this.explicitHydrogenCount=a);return!0});c$.getDefaultCount=h(c$,"getDefaultCount",function(a,b){switch(a){case 0:case -1:case -2:return-1;case 6:return b?3:4;case 8:case 16:return 2;case 7:return b?2:3;case 5:case 15:return 3;case 9:case 17:case 35:case 53:return 1}return-2},"~N,~B");n(c$,"getIndex",function(){return this.index});
h(c$,"setSymbol",function(a){this.symbol=a;this.isAromatic=a.equals(a.toLowerCase());this.elementDefined=this.hasSymbol=!0;if(a.equals("*"))return this.isAromatic=!1,this.elementNumber=-2,!0;if(a.equals("Xx"))return this.elementNumber=0,!0;this.aromaticAmbiguous=!1;if(a.equals("a")||a.equals("A"))return 0>this.elementNumber&&(this.elementNumber=-1),!0;this.isAromatic&&(a=a.substring(0,1).toUpperCase()+(1==a.length?"":a.substring(1)));this.elementNumber=JU.Elements.elementNumberFromSymbol(a,!0);return 0!=
this.elementNumber},"~S");n(c$,"getElementNumber",function(){return this.elementNumber});h(c$,"getAtomicMass",function(){return this.atomicMass});n(c$,"getAtomNumber",function(){return this.atomNumber});h(c$,"setAtomicMass",function(a){this.atomicMass=a},"~N");h(c$,"getCharge",function(){return this.charge});h(c$,"setCharge",function(a){this.charge=a},"~N");h(c$,"getMatchingAtomIndex",function(){return this.matchingIndex});h(c$,"getMatchingAtom",function(){return this.matchingNode});h(c$,"setMatchingAtom",
function(a,b){this.matchingNode=a;this.matchingIndex=b},"JU.Node,~N");h(c$,"setExplicitHydrogenCount",function(a){this.explicitHydrogenCount=a},"~N");h(c$,"setImplicitHydrogenCount",function(a){this.implicitHydrogenCount=a},"~N");h(c$,"setDegree",function(a){this.degree=a},"~N");h(c$,"setNonhydrogenDegree",function(a){this.nonhydrogenDegree=a},"~N");h(c$,"setValence",function(a){this.valence=a},"~N");h(c$,"setConnectivity",function(a){this.connectivity=a},"~N");h(c$,"setRingMembership",function(a){this.ringMembership=
a},"~N");h(c$,"setRingSize",function(a){this.ringSize=a;if(500==this.ringSize||600==this.ringSize)this.isAromatic=!0},"~N");h(c$,"setRingConnectivity",function(a){this.ringConnectivity=a},"~N");n(c$,"getModelIndex",function(){return this.component});n(c$,"getMoleculeNumber",function(){return this.component},"~B");n(c$,"getAtomSite",function(){return this.atomSite});n(c$,"getFormalCharge",function(){return this.charge});n(c$,"getIsotopeNumber",function(){return this.atomicMass});n(c$,"getAtomicAndIsotopeNumber",
function(){return JU.Elements.getAtomicAndIsotopeNumber(this.elementNumber,this.atomicMass)});n(c$,"getAtomName",function(){return null==this.bioAtomName?"":this.bioAtomName});n(c$,"getGroup3",function(){return null==this.residueName?"":this.residueName},"~B");n(c$,"getGroup1",function(){return null==this.residueChar?"":this.residueChar},"~S");h(c$,"addBond",function(a){this.bondCount>=this.bonds.length&&(this.bonds=JU.AU.doubleLength(this.bonds));this.bonds[this.bondCount]=a;this.bondCount++},"JS.SmilesBond");
h(c$,"setBondArray",function(){this.bonds.length>this.bondCount&&(this.bonds=JU.AU.arrayCopyObject(this.bonds,this.bondCount));null!=this.subAtoms&&this.subAtoms.length>this.nSubAtoms&&(this.subAtoms=JU.AU.arrayCopyObject(this.subAtoms,this.subAtoms.length));for(var a=0;a<this.bonds.length;a++){var b=this.bonds[a];this.isBioAtom&&17==b.order&&(b.order=112);b.atom1.index>b.atom2.index&&b.switchAtoms()}});n(c$,"getEdges",function(){return null!=this.parent?this.parent.getEdges():this.bonds});h(c$,"getBond",
function(a){return null!=this.parent?this.parent.getBond(a):0<=a&&a<this.bondCount?this.bonds[a]:null},"~N");n(c$,"getCovalentBondCount",function(){return this.getBondCount()});n(c$,"getBondCount",function(){return null!=this.parent?this.parent.getBondCount():this.bondCount});n(c$,"getCovalentBondCountPlusMissingH",function(){return this.getBondCount()+(this.isTopoAtom?0:this.missingHydrogenCount)});n(c$,"getTotalHydrogenCount",function(){return this.getCovalentHydrogenCount()+(this.isTopoAtom?0:
this.missingHydrogenCount)});n(c$,"getImplicitHydrogenCount",function(){return this.implicitHydrogenCount});h(c$,"getExplicitHydrogenCount",function(){return this.explicitHydrogenCount});h(c$,"getMatchingBondedAtom",function(a){if(null!=this.parent)return this.parent.getMatchingBondedAtom(a);if(a>=this.bondCount)return-1;a=this.bonds[a];return(a.atom1===this?a.atom2:a.atom1).matchingIndex},"~N");n(c$,"getBondedAtomIndex",function(a){return null!=this.parent?this.parent.getBondedAtomIndex(a):this.bonds[a].getOtherAtom(this).index},
"~N");n(c$,"getCovalentHydrogenCount",function(){if(0<=this.covalentHydrogenCount)return this.covalentHydrogenCount;if(null!=this.parent)return this.covalentHydrogenCount=this.parent.getCovalentHydrogenCount();for(var a=this.covalentHydrogenCount=0;a<this.bonds.length;a++)1==this.bonds[a].getOtherAtom(this).elementNumber&&this.covalentHydrogenCount++;return this.covalentHydrogenCount});n(c$,"getValence",function(){if(null!=this.parent)return this.parent.getValence();var a=this.valence;if(0>=a&&null!=
this.bonds)for(var b=this.bonds.length;0<=--b;)a+=this.bonds[b].getValence();return this.valence=a});n(c$,"getTotalValence",function(){return this.getValence()+(this.isTopoAtom?0:this.missingHydrogenCount)});h(c$,"getBondTo",function(a){if(null!=this.parent)return this.parent.getBondTo(a);for(var b,c=0;c<this.bonds.length;c++)if(null!=(b=this.bonds[c]))if(null==a?b.atom2===this:b.getOtherAtom(this)===a)return b;return null},"JS.SmilesAtom");h(c$,"getBondNotTo",function(a,b){for(var c,d=0;d<this.bonds.length;d++)if(null!=
(c=this.bonds[d])){var e=c.getOtherAtom(this);if(a!==e&&(b||1!=e.elementNumber))return c}return null},"JS.SmilesAtom,~B");n(c$,"isLeadAtom",function(){return this.$isLeadAtom});n(c$,"getOffsetResidueAtom",function(a,b){if(this.isBioAtom){if(0==b)return this.index;for(var c=0;c<this.bonds.length;c++)if(this.bonds[c].getAtomIndex1()==this.index&&96==this.bonds[c].order)return this.bonds[c].getOtherAtom(this).index}return-1},"~S,~N");n(c$,"getGroupBits",function(a){a.set(this.index)},"JU.BS");n(c$,"isCrossLinked",
function(a){return this.getBondTo(a).isHydrogen()},"JU.Node");n(c$,"getCrossLinkVector",function(a){for(var b=!1,c=0;c<this.bonds.length;c++)if(112==this.bonds[c].order){if(null==a)return!0;a.addLast(Integer.$valueOf(this.index));a.addLast(Integer.$valueOf(this.bonds[c].getOtherAtom(this).index));a.addLast(Integer.$valueOf(this.bonds[c].getOtherAtom(this).index));b=!0}return b},"JU.Lst,~B,~B");n(c$,"getBioStructureTypeName",function(){return null});n(c$,"getInsertionCode",function(){return this.insCode});
n(c$,"getResno",function(){return this.residueNumber});n(c$,"getChainID",function(){return 0});n(c$,"getChainIDStr",function(){return""});c$.getAtomLabel=h(c$,"getAtomLabel",function(a,b,c,d,e,f,g,j){var l=JU.Elements.elementSymbolFromNumber(a);g&&(l=l.toLowerCase(),6!=a&&(c=2147483647));return(2147483647==c||0!=b||0!=d||!Float.isNaN(e)||null!=j&&0<j.length?-1:JS.SmilesAtom.getDefaultCount(a,!1))==c?l:"["+(0>=b?"":""+b)+l+(null==j?"":j)+(1<f?"H"+f:1==f?"H":"")+(0>d&&-2147483648!=d?""+d:0<d?"+"+d:
"")+(Float.isNaN(e)?"":":"+Q(e))+"]"},"~N,~N,~N,~N,~N,~N,~B,~S");n(c$,"getBioSmilesType",function(){return this.bioType});h(c$,"isNucleic",function(){return"n"==this.bioType||"r"==this.bioType||"d"==this.bioType});n(c$,"isPurine",function(){return null!=this.residueChar&&this.isNucleic()&&0<="AG".indexOf(this.residueChar)});n(c$,"isPyrimidine",function(){return null!=this.residueChar&&this.isNucleic()&&0<="CTUI".indexOf(this.residueChar)});n(c$,"isDeleted",function(){return!1});n(c$,"findAtomsLike",
function(){return null},"~S");n(c$,"toString",function(){var a=null!=this.residueChar||null!=this.residueName?(null==this.residueChar?this.residueName:this.residueChar)+"."+this.bioAtomName:null!=this.bioAtomName&&-2147483648!=this.atomNumber?null:-1==this.elementNumber?"A":-2==this.elementNumber?"*":JU.Elements.elementSymbolFromNumber(this.elementNumber);if(null==a)return this.bioAtomName+" #"+this.atomNumber;this.isAromatic&&(a=a.toLowerCase());for(var b="",c=0;c<this.bondCount;c++)b+=this.bonds[c].getOtherAtom(this).index+
", ";return"["+a+"."+this.index+(0<=this.matchingIndex?"("+this.matchingNode+")":"")+"]"+b+"("+this.x+","+this.y+","+this.z+")"});n(c$,"getFloatProperty",function(a){return"property_atomclass"===a?this.atomClass:NaN},"~S");n(c$,"getNominalMass",function(){return this.getAtomicMass()});n(c$,"getCIPChirality",function(){return""},"~B");n(c$,"setCIPChirality",function(){},"~N");n(c$,"getCIPChiralityCode",function(){return 0});n(c$,"getXYZ",function(){return this});H(c$,"UNBRACKETED_SET","B, C, N, O, P, S, F, Cl, Br, I, *,")});
B("JS");C(["JU.Edge"],"JS.SmilesBond",["JS.InvalidSmilesException"],function(){c$=D(function(){this.atom2=this.atom1=null;this.isNot=!1;this.primitives=this.matchingBond=null;this.nPrimitives=0;this.bondsOr=null;this.nBondsOr=0;this.isConnection=!1;this.atropType=null;this.isChain=!1;E(this,arguments)},JS,"SmilesBond",JU.Edge);c$.getBondOrderString=h(c$,"getBondOrderString",function(a){switch(a){case 2:return"=";case 3:return"#";case 4:return"$";default:return""}},"~N");c$.getBondTypeFromCode=h(c$,
"getBondTypeFromCode",function(a){switch(a){case ".":return 0;case "-":return 1;case "=":return 2;case "#":return 3;case "$":return 4;case ":":return 17;case "/":return 1025;case "\\":return 1041;case "^":return 65537;case "`":return 65538;case "@":return 65;case "~":return 81;case "+":return 96}return-1},"~S");h(c$,"set",function(a){this.order=a.order;this.isNot=a.isNot;this.primitives=a.primitives;this.nPrimitives=a.nPrimitives;this.bondsOr=a.bondsOr;this.nBondsOr=a.nBondsOr},"JS.SmilesBond");h(c$,
"setAtropType",function(a){this.atropType=s(-1,[P(a/10)-1,a%10-1])},"~N");h(c$,"setPrimitive",function(a){a=this.primitives[a];this.order=a.order;this.isNot=a.isNot;this.atropType=a.atropType;return a},"~N");h(c$,"addBondOr",function(){null==this.bondsOr&&(this.bondsOr=Array(2));if(this.nBondsOr>=this.bondsOr.length){var a=Array(2*this.bondsOr.length);System.arraycopy(this.bondsOr,0,a,0,this.bondsOr.length);this.bondsOr=a}a=new JS.SmilesBond(null,null,-1,!1);this.bondsOr[this.nBondsOr]=a;this.nBondsOr++;
return a});h(c$,"addPrimitive",function(){null==this.primitives&&(this.primitives=Array(2));if(this.nPrimitives>=this.primitives.length){var a=Array(2*this.primitives.length);System.arraycopy(this.primitives,0,a,0,this.primitives.length);this.primitives=a}a=new JS.SmilesBond(null,null,-1,!1);this.primitives[this.nPrimitives]=a;this.nPrimitives++;return a});n(c$,"toString",function(){return this.atom1+" -"+(this.isNot?"!":"")+this.order+"- "+this.atom2});F(c$,function(a,b,c,d){M(this,JS.SmilesBond,
[]);this.set2(c,d);this.set2a(a,b)},"JS.SmilesAtom,JS.SmilesAtom,~N,~B");h(c$,"set2",function(a,b){this.order=a;this.isNot=b},"~N,~B");h(c$,"set2a",function(a,b){null!=a&&(this.atom1=a,a.addBond(this));null!=b&&(this.atom2=b,b.isBioAtomWild&&this.atom1.isBioAtomWild&&(this.order=96),b.isFirst=!1,b.addBond(this))},"JS.SmilesAtom,JS.SmilesAtom");h(c$,"setAtom2",function(a){this.atom2=a;null!=this.atom2&&(a.addBond(this),this.isConnection=!0)},"JS.SmilesAtom,JS.SmilesSearch");h(c$,"isFromPreviousTo",
function(a){return!this.isConnection&&this.atom2===a},"JS.SmilesAtom");c$.isBondType=h(c$,"isBondType",function(a,b,c){if(">"==a)return 1;if(0>"-=#$:/\\.~^`+!,&;@".indexOf(a))return 0;if(!b&&0>"-=#$:/\\.~^`".indexOf(a))throw new JS.InvalidSmilesException("SMARTS bond type "+a+" not allowed in SMILES");switch(a){case "~":return c?0:1;case "^":case "`":return-1;default:return 1}},"~S,~B,~B");h(c$,"getBondType",function(){return this.order});h(c$,"getValence",function(){return this.order&7});h(c$,"getOtherAtom",
function(a){return this.atom1===a?this.atom2:this.atom1},"JS.SmilesAtom");n(c$,"getAtomIndex1",function(){return this.atom1.index});n(c$,"getAtomIndex2",function(){return this.atom2.index});n(c$,"getCovalentOrder",function(){return this.order});n(c$,"getOtherAtomNode",function(a){return a===this.atom1?this.atom2:a===this.atom2||null==a?this.atom1:null},"JU.Node");n(c$,"isCovalent",function(){return 112!=this.order});n(c$,"isHydrogen",function(){return 112==this.order});h(c$,"switchAtoms",function(){var a=
this.atom1;this.atom1=this.atom2;this.atom2=a;switch(this.order){case 65537:this.order=65538;break;case 65538:this.order=65537;break;case 1025:this.order=1041;break;case 1041:this.order=1025}});H(c$,"TYPE_UNKNOWN",-1,"TYPE_NONE",0,"TYPE_AROMATIC",17,"TYPE_RING",65,"TYPE_ANY",81,"TYPE_BIO_SEQUENCE",96,"TYPE_BIO_CROSSLINK",112,"ALL_BONDS","-=#$:/\\.~^`+!,&;@","SMILES_BONDS","-=#$:/\\.~^`")});B("JS");C(null,"JS.SmilesMeasure",["JU.PT"],function(){c$=D(function(){this.search=null;this.index=this.type=
this.nPoints=0;this.isNot=!1;this.points=this.minmax=this.indices=null;E(this,arguments)},JS,"SmilesMeasure");I(c$,function(){this.indices=s(4,0);this.points=Array(4)});F(c$,function(a,b,c,d,e){this.search=a;this.type=Math.min(4,Math.max(c,2));this.index=b;this.isNot=d;this.minmax=e;for(a=e.length-2;0<=a;a-=2)e[a]>e[a+1]&&(b=e[a+1],e[a+1]=e[a],e[a]=b)},"JS.SmilesSearch,~N,~N,~B,~A");h(c$,"addPoint",function(a){if(this.nPoints==this.type)return!1;if(0==this.nPoints)for(var b=1;b<this.type;b++)this.indices[b]=
a+b;this.indices[this.nPoints++]=a;return!0},"~N");h(c$,"check",function(){for(var a=0;a<this.type;a++){var b=this.search.patternAtoms[this.indices[a]].getMatchingAtomIndex();this.points[a]=this.search.targetAtoms[b]}b=0;switch(this.type){case 2:b=this.points[0].distance(this.points[1]);break;case 3:this.search.v.vA.sub2(this.points[0],this.points[1]);this.search.v.vB.sub2(this.points[2],this.points[1]);b=this.search.v.vA.angle(this.search.v.vB)/0.017453292;break;case 4:b=JS.SmilesMeasure.setTorsionData(this.points[0],
this.points[1],this.points[2],this.points[3],this.search.v,!0)}for(a=this.minmax.length-2;0<=a;a-=2)if(b>=this.minmax[a]&&b<=this.minmax[a+1])return!this.isNot;return this.isNot});c$.setTorsionData=h(c$,"setTorsionData",function(a,b,c,d,e,f){e.vTemp1.sub2(a,b);e.vTemp2.sub2(d,c);if(!f)return 0;e.vNorm2.sub2(b,c);e.vNorm2.normalize();e.vTemp1.cross(e.vTemp1,e.vNorm2);e.vTemp1.normalize();e.vTemp2.cross(e.vTemp2,e.vNorm2);e.vTemp2.normalize();e.vNorm3.cross(e.vTemp1,e.vTemp2);return e.vTemp1.angle(e.vTemp2)/
0.017453292*(0>e.vNorm2.dot(e.vNorm3)?1:-1)},"JU.T3,JU.T3,JU.T3,JU.T3,JS.VTemp,~B");n(c$,"toString",function(){for(var a="(."+"__dat".charAt(this.type)+this.index+":"+JU.PT.toJSON(null,this.minmax)+") for",b=0;b<this.type;b++)a+=" "+(b>=this.nPoints?"?":""+this.indices[b]);return a});H(c$,"TYPES","__dat","radiansPerDegree",0.017453292519943295)});B("JS");C(["java.util.Hashtable"],"JS.SmilesParser","java.lang.Character $.Float JU.Lst $.PT $.SB JS.InvalidSmilesException $.SmilesAtom $.SmilesBond $.SmilesMeasure $.SmilesSearch $.SmilesStereo JU.Elements $.Logger".split(" "),
function(){c$=D(function(){this.htMeasures=this.connections=null;this.flags=0;this.isBioSequence=this.isSmarts=!1;this.bioType="\x00";this.componentParenCount=this.componentCount=this.branchLevel=this.braceCount=0;this.ignoreStereochemistry=!1;this.bondDirectionPaired=!0;this.isTarget=!1;E(this,arguments)},JS,"SmilesParser");I(c$,function(){this.connections=new java.util.Hashtable;this.htMeasures=new java.util.Hashtable});c$.newSearch=h(c$,"newSearch",function(a,b,c){return(new JS.SmilesParser(b,
c)).parse(a)},"~S,~B,~B");F(c$,function(a,b){this.isSmarts=a;this.isTarget=b},"~B,~B");h(c$,"parse",function(a){if(null==a)throw new JS.InvalidSmilesException("expression must not be null");var b=new JS.SmilesSearch;0<=a.indexOf("$(select")&&(a=this.parseNested(b,a,"select"));var c=s(1,0);a=JS.SmilesParser.extractFlags(a,c);this.flags=c[0];this.ignoreStereochemistry=32==(this.flags&32);b.setFlags(this.flags);0<=a.indexOf("$")&&(a=this.parseVariables(a));this.isSmarts&&0<=a.indexOf("[$")&&(a=this.parseVariableLength(a));
if(0>a.indexOf("||"))return this.getSubsearch(b,a,this.flags);a=JU.PT.split(a,"||");c="";b.subSearches=Array(a.length);for(var d=0;d<a.length;d++){var e="|"+a[d]+"|";0>c.indexOf(e)&&(b.subSearches[d]=this.getSubsearch(b,a[d],this.flags),c+=e)}return b},"~S");h(c$,"parseVariableLength",function(a){for(var b=new JU.SB,c=a.length-1,d=0,e=!1,f=0;f<c;f++)switch(a.charAt(f)){case "(":d++;break;case ")":d--;break;case "|":0<d&&(e=!0,"|"==a.charAt(f+1)&&(a=a.substring(0,f)+a.substring(f+1),c--))}if(0<=a.indexOf("||")){c=
JU.PT.split(a,"||");for(f=0;f<c.length;f++)b.append("||").append(this.parseVariableLength(c[f]))}else{c=-1;d=s(1,0);for(f=null;0<=(c=a.indexOf("[$",c+1));){var g=c,j=-2147483648,l=-2147483648,c=JS.SmilesParser.getDigits(a,c+2,d),j=d[0];-2147483648!=j&&"-"==JS.SmilesParser.getChar(a,c)&&(c=JS.SmilesParser.getDigits(a,c+1,d),l=d[0]);if("("==JS.SmilesParser.getChar(a,c)&&(f=JS.SmilesParser.getSubPattern(a,g,"["),f.endsWith(")"))){var h=g+f.length+2,k=JS.SmilesParser.getSubPattern(a,c,"("),m=c;JS.SmilesParser.getSubPattern(a,
c,"[");c+=1+k.length;if(0<=k.indexOf(":")&&0>k.indexOf("|")){for(var n=0,p=k.length,q=-1,f=0;f<p;f++)switch(k.charAt(f)){case "[":case "(":n++;break;case ")":case "]":n--;break;case ".":0<=q&&0==n&&(p=f);break;case ":":0>q&&0==n&&(q=f)}0<q&&(k=k.substring(0,q)+"("+k.substring(q,p)+")"+k.substring(p))}if(-2147483648==j){if(f=k.indexOf("|"),0<=f)return this.parseVariableLength(a.substring(0,g)+"[$1"+a.substring(m,m+f+1)+")]"+a.substring(h)+"||"+a.substring(0,g)+"[$1("+a.substring(m+f+2)+a.substring(h))}else{-2147483648==
l&&(l=j);0<=k.indexOf("|")&&(k="[$("+k+")]");for(f=j;f<=l;f++){j=new JU.SB;j.append("||").append(a.substring(0,g));for(m=0;m<f;m++)j.append(k);j.append(a.substring(h));b.appendSB(j)}}}}}return e?this.parseVariableLength(b.substring(2)):2>b.length()?a:b.substring(2)},"~S");h(c$,"getSubsearch",function(a,b,c){this.htMeasures=new java.util.Hashtable;var d=new JS.SmilesSearch;d.setTop(a);d.isSmarts=this.isSmarts;d.pattern=b;d.setFlags(c);0<=b.indexOf("$(")&&(b=this.parseNested(d,b,""));this.parseSmiles(d,
b,null,!1);if(0!=this.braceCount)throw new JS.InvalidSmilesException("unmatched '{'");if(!this.connections.isEmpty())throw new JS.InvalidSmilesException("Open connection");d.set();if(this.isSmarts)for(a=d.ac;0<=--a;)this.checkNested(d,d.patternAtoms[a],c);else this.isBioSequence||(d.elementCounts[1]=d.getMissingHydrogenCount());!this.ignoreStereochemistry&&!this.isTarget&&this.fixChirality(d);return d},"JS.SmilesSearch,~S,~N");h(c$,"checkNested",function(a,b,c){if(0<b.iNested){var d=a.getNested(b.iNested);
if(L(d,String)){if(d.startsWith("select"))return;"~"!=d.charAt(0)&&"\x00"!=b.bioType&&(d="~"+b.bioType+"~"+d);d=this.getSubsearch(a,d,c);0<d.ac&&d.patternAtoms[0].selected&&(b.selected=!0);a.setNested(b.iNested,d)}}for(d=0;d<b.nSubAtoms;d++)this.checkNested(a,b.subAtoms[d],c)},"JS.SmilesSearch,JS.SmilesAtom,~N");h(c$,"fixChirality",function(a){for(var b=a.ac;0<=--b;){var c=a.patternAtoms[b];null!=c.stereo&&c.stereo.fixStereo(c)}},"JS.SmilesSearch");h(c$,"parseSmiles",function(a,b,c,d){var e=s(1,0),
f=0,g,j=null,l=!1,h=!1;a:for(;null!=b&&0!=b.length;){var k=0;null==c&&(k=this.checkBioType(b,0),k==b.length&&(b+="*"),this.isBioSequence&&(a.needAromatic=a.top.needAromatic=!1));g=JS.SmilesParser.getChar(b,k);var m=this.checkBrace(a,g,"{");m&&(g=JS.SmilesParser.getChar(b,++k));if("("==g)if(j=JS.SmilesParser.getSubPattern(b,k,"("),g="."==JS.SmilesParser.getChar(b,k+1),null==c){if(g||!this.isSmarts)throw new JS.InvalidSmilesException("No previous atom for measure");a.haveComponents=!0;do this.componentCount++,
this.componentParenCount++,g=JS.SmilesParser.getChar(b=b.substring(1),0);while("("==g);if(!m&&!0==(m=this.checkBrace(a,g,"{")))g=JS.SmilesParser.getChar(b=b.substring(1),0)}else l=h=!1,j.startsWith(".")?(this.parseMeasure(a,j.substring(1),c),l=!0):0==j.length&&this.isBioSequence?c.notCrossLinked=!0:(this.branchLevel++,this.parseSmiles(a,j,c,!0),h=!0,this.branchLevel--),k=j.length+2,g=JS.SmilesParser.getChar(b,k),"}"==g&&this.checkBrace(a,g,"}")&&k++,g="\x00";if("\x00"!=g){f=k;b:for(;"\x00"!=g;){switch(JS.SmilesBond.isBondType(g,
this.isSmarts,this.isBioSequence)){case 0:break b;case -1:if(!((JU.PT.isDigit(JS.SmilesParser.getChar(b,++k))&&0<k++?JU.PT.isDigit(JS.SmilesParser.getChar(b,k++)):1)&&"-"==(g=JS.SmilesParser.getChar(b,k))))throw new JS.InvalidSmilesException("malformed atropisomerism bond ^nn-  or ^^nn-");continue}g=JS.SmilesParser.getChar(b,++k)}g=JS.SmilesParser.getChar(b,k);if(")"==g){switch(JS.SmilesParser.getChar(b,++k)){case "\x00":case ")":case ".":if(b=b.substring(k),this.componentParenCount--,0<=this.componentParenCount)continue a}throw new JS.InvalidSmilesException("invalid continuation after component grouping (SMARTS).(SMARTS)");
}j=this.parseBond(a,null,b.substring(f,k),null,c,!1,d,k-f,e);m&&-1!=j.order&&(g=JS.SmilesParser.getChar(b,k=f));this.checkBrace(a,g,"{")&&(g=JS.SmilesParser.getChar(b,++k));switch(g){case "~":0==j.order&&(k=this.checkBioType(b,k),k==b.length&&(b+="*"));break;case "(":do this.componentCount++,this.componentParenCount++,g=JS.SmilesParser.getChar(b,++k);while("("==g);break;case "\x00":if(0==j.order)return}m=JU.PT.isDigit(g)||"%"==g;f=!m&&("_"==g||"["==g||"*"==g||JU.PT.isLetter(g));if(m){if(l||h)throw new JS.InvalidSmilesException("connection number must immediately follow its connecting atom");
k=JS.SmilesParser.getRingNumber(b,k,g,e);this.parseConnection(a,e[0],c,j)}else if(f)switch(l=h=!1,g){case "[":case "_":m=JS.SmilesParser.getSubPattern(b,k,g);k+=m.length+("["==g?2:0);this.isBioSequence&&("["==g&&0>m.indexOf(".")&&0>m.indexOf("_"))&&(m+=".0");c=this.parseAtom(a,null,m,c,j,"["==g,!1,d);c.hasSubpattern=!0;-1!=j.order&&0!=j.order&&this.setBondAtom(j,null,c,a);break;default:m=!this.isBioSequence&&JU.PT.isUpperCase(g)?JS.SmilesParser.getChar(b,k+1):"\x00";if("X"!=g||"x"!=m)if(!JU.PT.isLowerCase(m)||
0==JU.Elements.elementNumberFromSymbol(b.substring(k,k+2),!0))m="\x00";"\x00"!=m&&0<="NA CA BA PA SC AC".indexOf(b.substring(k,k+2))&&(m="\x00");g=JU.PT.isUpperCase(g)&&JU.PT.isLowerCase(m)?2:1;c=this.parseAtom(a,null,b.substring(k,k+g),c,j,!1,!1,d);k+=g}else throw new JS.InvalidSmilesException("Unexpected character: "+JS.SmilesParser.getChar(b,k));g=JS.SmilesParser.getChar(b,k);"}"==g&&this.checkBrace(a,g,"}")&&k++}b=b.substring(k);d=!1}},"JS.SmilesSearch,~S,JS.SmilesAtom,~B");h(c$,"parseConnection",
function(a,b,c,d){b=Integer.$valueOf(b);var e=this.connections.get(b);if(null==e)this.connections.put(b,d),a.top.ringCount++;else{this.connections.remove(b);switch(d.order){case -1:d.order=-1!=e.order?e.order:this.isSmarts||c.isAromatic&&e.atom1.isAromatic?81:1;break;case 1025:d.order=1041;break;case 1041:d.order=1025}if(-1!=e.order&&e.order!=d.order||c===e.atom1||null!=e.atom1.getBondTo(c))throw new JS.InvalidSmilesException("Bad connection type or atom");e.set(d);c.bondCount--;e.setAtom2(c,a)}},
"JS.SmilesSearch,~N,JS.SmilesAtom,JS.SmilesBond");h(c$,"setBondAtom",function(a,b,c,d){a.set2a(b,c);null!=d&&(2==a.order&&null!=a.atom1&&null!=a.atom2&&a.atom1.isAromatic&&a.atom2.isAromatic&&0==(this.flags&512))&&d.setFlags(this.flags|=512)},"JS.SmilesBond,JS.SmilesAtom,JS.SmilesAtom,JS.SmilesSearch");c$.getRingNumber=h(c$,"getRingNumber",function(a,b,c,d){switch(c){case "%":if("("==JS.SmilesParser.getChar(a,b+1)){if(a=JS.SmilesParser.getSubPattern(a,b+1,"("),JS.SmilesParser.getDigits(a,0,d),b+=
a.length+3,0>d[0])throw new JS.InvalidSmilesException("Invalid number designation: "+a);}else if(b+3<=a.length&&(b=JS.SmilesParser.getDigits(a.substring(0,b+3),b+1,d)),10>d[0])throw new JS.InvalidSmilesException("Two digits must follow the % sign");a=d[0];break;default:a=c.charCodeAt(0)-48,b++}d[0]=a;return b},"~S,~N,~S,~A");h(c$,"checkBioType",function(a,b){if(this.isBioSequence="~"==a.charAt(b)){b++;this.bioType="*";var c=JS.SmilesParser.getChar(a,2);if("~"==c&&("*"==(c=a.charAt(1))||JU.PT.isLowerCase(c)))this.bioType=
c,b=3}return b},"~S,~N");h(c$,"parseMeasure",function(a,b,c){for(var d=b.indexOf(":"),e=0>d?b:b.substring(0,d);0!=d;){var f=e.length;1==f&&(e+="0");var g=this.htMeasures.get(e);if(null==g==0>d||0==f)break;try{if(0<d){var j="__dat".indexOf(e.charAt(0));if(2>j)break;var l=s(1,0);JS.SmilesParser.getDigits(e,1,l);var h=l[0];b=b.substring(d+1);var k=b.startsWith("!");k&&(b=b.substring(1));var m=b.startsWith("-");m&&(b=b.substring(1));b=JU.PT.rep(b,"-",",");b=JU.PT.rep(b,",,",",-");m&&(b="-"+b);var n=JU.PT.split(b,
",");if(1==n.length%2||k&&2!=n.length)break;for(var p=N(n.length,0),q=n.length;0<=--q&&!Float.isNaN(p[q]=JU.PT.fVal(n[q])););if(0<=q)break;g=new JS.SmilesMeasure(a,h,j,k,p);a.measures.addLast(g);0<h?this.htMeasures.put(e,g):0==h&&JU.Logger.debugging&&JU.Logger.debug("measure created: "+g)}else{if(!g.addPoint(c.index))break;g.nPoints==g.type&&(this.htMeasures.remove(e),JU.Logger.debugging&&JU.Logger.debug("measure created: "+g));return}if(!g.addPoint(c.index))break}catch(r){if(G(r,NumberFormatException))break;
else throw r;}return}throw new JS.InvalidSmilesException("invalid measure: "+b);},"JS.SmilesSearch,~S,JS.SmilesAtom");h(c$,"checkBrace",function(a,b,c){switch(b){case "{":if(b!=c)break;this.braceCount++;return a.top.haveSelected=!0;case "}":if(b!=c)break;if(0<this.braceCount)return this.braceCount--,!0;break;default:return!1}throw new JS.InvalidSmilesException("Unmatched '}'");},"JS.SmilesSearch,~S,~S");h(c$,"parseNested",function(a,b,c){var d;for(c="$("+c;0<=(d=b.lastIndexOf(c));){var e=JS.SmilesParser.getSubPattern(b,
d+1,"("),f=d+e.length+3,g=b.substring(f);b=b.substring(0,d);var j="";if(b.endsWith("]"))throw new JS.InvalidSmilesException("$(...) must be enclosed in brackets: "+b+"$("+e+")");if(1<d&&2==c.length&&1<(f=b.length)&&0>",;&![".indexOf(b.substring(f-1)))j="&";1<g.length&&0>",;&!)]".indexOf(g.charAt(0))&&(g="&"+g);b=b+j+"_"+a.top.addNested(e)+"_"+g}return b},"JS.SmilesSearch,~S,~S");h(c$,"parseVariables",function(a){var b=new JU.Lst,c=new JU.Lst,d,e=0,f=-1;for(JU.Logger.debugging&&JU.Logger.info(a);0<=
(d=a.indexOf("$",e))&&"("!=JS.SmilesParser.getChar(a,d+1);){e=JS.SmilesParser.skipTo(a,d,"=");if(e<=d+1||'"'!=JS.SmilesParser.getChar(a,e+1))break;d=a.substring(d,e);if(0<d.lastIndexOf("$")||0<d.indexOf("]"))throw new JS.InvalidSmilesException("Invalid variable name: "+d);f=JS.SmilesParser.getSubPattern(a,e+1,'"');b.addLast("["+d+"]");c.addLast(f);e+=f.length+2;e=JS.SmilesParser.skipTo(a,e,";");f=++e}if(0>f)return a;a=a.substring(f);for(e=b.size();0<=--e;)d=b.get(e),f=c.get(e),f.equals(d)||(a=JU.PT.rep(a,
d,f));JU.Logger.debugging&&JU.Logger.info(a);return a},"~S");h(c$,"parseAtom",function(a,b,c,d,e,f,g,j){if(null==c||0==c.length)throw new JS.InvalidSmilesException("Empty atom definition");var l=new JS.SmilesAtom;0<this.componentParenCount&&(l.component=this.componentCount);null==b&&a.appendAtom(l);if(!this.checkLogic(a,c,l,null,d,g,j,null)){var h=s(1,0);this.isBioSequence&&1==c.length&&(c+=".0");var k=c.charAt(0),m=0,n=!1;if(this.isSmarts&&"!"==k){k=JS.SmilesParser.getChar(c,++m);if("\x00"==k)throw new JS.InvalidSmilesException("invalid '!'");
l.not=n=!0}var p=c.indexOf(".");if(0<=p){l.isBioResidue=!0;var q=c.substring(m,p);c=c.substring(p+1).toUpperCase();var r=q.length;if(0<=(p=q.indexOf("^")))p==r-2&&(k=q.charAt(r-1),"*"!=k&&(l.insCode=k)),q=q.substring(0,p);if(0<=(p=q.indexOf("#")))JS.SmilesParser.getDigits(q,p+1,h),l.residueNumber=h[0],q=q.substring(0,p);0==q.length&&(q="*");1<q.length?l.residueName=q.toUpperCase():q.equals("*")||(l.residueChar=q);q=c;if(0<=(p=q.indexOf("#")))JS.SmilesParser.getDigits(q,p+1,h),l.elementNumber=h[0],
q=q.substring(0,p);0==q.length?q="*":q.equals("0")&&(q="\x00");q.equals("*")?l.isBioAtomWild=!0:l.setAtomName(q);k="\x00"}l.setBioAtom(this.bioType);for(p=-2147483648;"\x00"!=k;){l.setAtomName(this.isBioSequence?"\x00":"");if(JU.PT.isDigit(k)){m=JS.SmilesParser.getDigits(c,m,h);k=h[0];if(-2147483648==k)throw new JS.InvalidSmilesException("Non numeric atomic mass");"?"==JS.SmilesParser.getChar(c,m)&&(m++,k=-k);if(l.elementDefined)throw new JS.InvalidSmilesException('atom mass must precede atom symbol or be separated from it with ";"');
l.setAtomicMass(k)}else switch(k){case '"':k=JU.PT.getQuotedStringAt(c,m);m+=k.length+2;l.atomType=k;break;case "_":m=JS.SmilesParser.getDigits(c,m+1,h)+1;if(-2147483648==h[0])throw new JS.InvalidSmilesException("Invalid SEARCH primitive: "+c.substring(m));l.iNested=h[0];if(!f)throw new JS.InvalidSmilesException("nesting must appear in [...]: $("+a.getNested(h[0])+")");if(this.isBioSequence&&m!=c.length)throw new JS.InvalidSmilesException("invalid characters: "+c.substring(m));break;case "=":m=JS.SmilesParser.getDigits(c,
m+1,h);l.jmolIndex=h[0];break;case "#":k="-"==c.charAt(m+1);m=JS.SmilesParser.getDigits(c,m+(k?2:1),h);k?l.atomNumber=h[0]:l.elementNumber=h[0];break;case "-":case "+":m=this.checkCharge(c,m,l);break;case "@":null==a.stereo&&(a.stereo=JS.SmilesStereo.newStereo(null));m=JS.SmilesStereo.checkChirality(c,m,a.patternAtoms[l.index]);break;case ":":m=JS.SmilesParser.getDigits(c,++m,h);if(-2147483648==h[0])throw new JS.InvalidSmilesException("Invalid atom class");l.atomClass=h[0];break;default:var q=JS.SmilesParser.getChar(c,
m+1),r=m+(JU.PT.isLowerCase(q)&&(!f||!JU.PT.isDigit(JS.SmilesParser.getChar(c,m+2)))?2:1),u=c.substring(m+1,r),y=Character.toUpperCase(k)+u,v=!0;f&&JU.PT.isLetter(k)&&(!n&&(g?b:l).hasSymbol?v=!1:"H"==k?v=1==c.length||!JU.PT.isDigit(q):JU.PT.isDigit(q)?v=!1:!y.equals("A")&&!y.equals("Xx")&&(v=("h"==k?2==r:!0)&&0<JU.Elements.elementNumberFromSymbol(y,!0),!v&&2==r&&(u="",y=y.substring(0,1),v=0<JU.Elements.elementNumberFromSymbol(y,!0))));if(v){if(!f&&!this.isSmarts&&!this.isBioSequence&&!JS.SmilesAtom.allowSmilesUnbracketed(y)||
!l.setSymbol(y=k+u))throw new JS.InvalidSmilesException("Invalid atom symbol: "+y);g&&(b.hasSymbol=!0);m+=y.length}else switch(m=JS.SmilesParser.getDigits(c,m+1,h),r=h[0],k){default:throw new JS.InvalidSmilesException("Invalid SEARCH primitive: "+c.substring(m));case "D":l.setDegree(-2147483648==r?1:r);break;case "d":l.setNonhydrogenDegree(-2147483648==r?1:r);break;case "H":p=-2147483648==r?1:r;break;case "h":l.setImplicitHydrogenCount(-2147483648==r?-1:r);break;case "R":-2147483648==r&&(r=-1);l.setRingMembership(r);
a.top.needRingData=!0;break;case "r":if(-2147483648==r)r=-1,l.setRingMembership(r);else{l.setRingSize(r);switch(r){case 500:r=5;break;case 600:r=6}r>a.ringDataMax&&(a.ringDataMax=r)}a.top.needRingData=!0;break;case "v":l.setValence(-2147483648==r?1:r);break;case "X":l.setConnectivity(-2147483648==r?1:r);break;case "x":l.setRingConnectivity(-2147483648==r?-1:r),a.top.needRingData=!0}}k=JS.SmilesParser.getChar(c,m);if(n&&"\x00"!=k)throw new JS.InvalidSmilesException("'!' may only involve one primitive.");
}-2147483648==p&&f&&(p=-2147483647);l.setExplicitHydrogenCount(p);a.patternAtoms[l.index].setExplicitHydrogenCount(p)}0<this.braceCount&&(l.selected=!0);null!=b&&b.addSubAtom(l,g);null!=d&&0==e.order&&(l.notBondedIndex=d.index);null!=d&&0!=e.order&&(-1==e.order&&(e.order=this.isBioSequence&&j?112:this.isSmarts||d.isAromatic&&l.isAromatic?81:1),f||this.setBondAtom(e,null,l,a),0==this.branchLevel&&(17==e.order||112==e.order)&&this.branchLevel++);0==this.branchLevel&&(a.lastChainAtom=l);return l},"JS.SmilesSearch,JS.SmilesAtom,~S,JS.SmilesAtom,JS.SmilesBond,~B,~B,~B");
h(c$,"checkCharge",function(a,b,c){var d=a.length,e=a.charAt(b),f=1;++b;if(b<d){var g=a.charAt(b);if(JU.PT.isDigit(g)){if(d=s(1,0),b=JS.SmilesParser.getDigits(a,b,d),f=d[0],-2147483648==f)throw new JS.InvalidSmilesException("Non numeric charge");}else for(;b<d&&a.charAt(b)==e;)b++,f++}c.setCharge("+"==e?f:-f);return b},"~S,~N,JS.SmilesAtom");h(c$,"parseBond",function(a,b,c,d,e,f,g,j,h){var n;if(0<j)switch(n=c.charAt(0)){case ">":if(!c.equals(">>")){j=-1;break}case ".":if(null==d&&null==b)return this.isBioSequence=
"~"==JS.SmilesParser.getChar(c,1),new JS.SmilesBond(null,null,0,!1);j=-1;break;case "+":null!=b&&(j=-1)}else n="\x00";d=null==b?null==d?new JS.SmilesBond(e,null,this.isBioSequence&&null!=e?g?112:96:-1,!1):d:f?b.addPrimitive():b.addBondOr();if(0<j&&!this.checkLogic(a,c,null,d,e,f,!1,h)){if(f="!"==n)if(n=JS.SmilesParser.getChar(c,1),"\x00"==n||"!"==n)throw new JS.InvalidSmilesException("invalid '!'");g=JS.SmilesBond.getBondTypeFromCode(n);65==g&&(a.top.needRingMemberships=!0);if(null==e&&0!=g)throw new JS.InvalidSmilesException("Bond without a previous atom");
switch(g){case 65537:case 65538:(j=c.length)<(f?3:2)||"-"!=c.charAt(j-1)?j=0:j==(f?3:2)?d.setAtropType(22):(JS.SmilesParser.getDigits(c,f?2:1,h),d.setAtropType(h[0]));a.haveBondStereochemistry=!0;break;case 1025:case 1041:this.bondDirectionPaired=!this.bondDirectionPaired;a.haveBondStereochemistry=!0;break;case 2:a.top.nDouble++;case 1:e.isAromatic&&(a.top.needRingData=!0)}d.set2(g,f);this.isBioSequence&&null!=b&&b.set2(g,f)}if(-1==j)throw new JS.InvalidSmilesException("invalid bond:"+n);return d},
"JS.SmilesSearch,JS.SmilesBond,~S,JS.SmilesBond,JS.SmilesAtom,~B,~B,~N,~A");h(c$,"checkLogic",function(a,b,c,d,e,f,g,j){var h=b.lastIndexOf("!");null!=c&&(c.pattern=b);for(;0<h;)0>",;&!".indexOf(b.charAt(h-1))&&(b=b.substring(0,h)+"&"+b.substring(h)),h=b.lastIndexOf("!",h-1);var h=b.indexOf(","),n=b.length,k="&";a:for(;;){var m=0<h;if(m&&!this.isSmarts||0==h)break;h=b.indexOf(";");if(0<=h){if(!this.isSmarts||0==h)break;m?(k=";",m=!1):b=b.$replace(";","&")}var s=0;if(m)for(b+=",";0<(h=b.indexOf(",",
s))&&h<=n;){s=b.substring(s,h);if(0==s.length)throw new JS.InvalidSmilesException("missing "+(null==d?"atom":"bond")+" token");null==d?this.parseAtom(a,c,s,null,null,!0,!1,g):this.parseBond(a,d,s,null,e,!1,!1,s.length,j);s=h+1}else if(0<=(h=b.indexOf(k))||null!=d&&1<n&&!f){if(0==h||null==d&&!this.isSmarts)break;if(null!=d&&0>h&&1<n){f=new JU.SB;for(m=0;m<n;){var p=b.charAt(m++);f.appendC(p);switch(p){case "!":if(!this.isSmarts)break a;continue;case "^":case "`":for(;"-"!=(p=b.charAt(m++))&&"\x00"!=
p;)f.appendC(p);f.appendC("-")}if(m<n){if(!this.isSmarts)break a;f.append(k)}}b=f.toString();n=b.length}for(b+=k;0<(h=b.indexOf(k,s))&&h<=n;)s=b.substring(s,h),null==d?this.parseAtom(a,c,s,null,null,!0,!0,g):this.parseBond(a,this.isSmarts?d:null,s,this.isSmarts?null:d,e,!0,!1,s.length,j),s=h+1}else return!1;return!0}p=b.charAt(h);throw new JS.InvalidSmilesException((this.isSmarts?"invalid placement for '"+p+"'":"["+p+"] notation only valid with SMARTS, not SMILES,")+" in "+b);},"JS.SmilesSearch,~S,JS.SmilesAtom,JS.SmilesBond,JS.SmilesAtom,~B,~B,~A");
c$.getSubPattern=h(c$,"getSubPattern",function(a,b,c){var d,e=1;switch(c){case "[":d="]";break;case '"':case "%":case "/":d=c;break;case "(":d=")";break;default:d=c,e=0}for(var f=a.length,g=1,j=b+1;j<f;j++){var h=a.charAt(j);if(h==d){if(g--,0==g)return a.substring(b+e,j+1-e)}else h==c&&g++}throw new JS.InvalidSmilesException("Unmatched "+c);},"~S,~N,~S");c$.getChar=h(c$,"getChar",function(a,b){return b<a.length?a.charAt(b):"\x00"},"~S,~N");c$.getDigits=h(c$,"getDigits",function(a,b,c){for(var d=b,
e=a.length;d<e&&JU.PT.isDigit(a.charAt(d));)d++;if(d>b)try{return c[0]=Integer.parseInt(a.substring(b,d)),d}catch(f){if(!G(f,NumberFormatException))throw f;}c[0]=-2147483648;return d},"~S,~N,~A");c$.skipTo=h(c$,"skipTo",function(a,b,c){for(var d;(d=JS.SmilesParser.getChar(a,++b))!=c&&"\x00"!=d;);return"\x00"==d?-1:b},"~S,~N,~S");c$.cleanPattern=h(c$,"cleanPattern",function(a){a=JU.PT.replaceAllCharacters(a," \t\n\r","");a=JU.PT.rep(a,"^^","`");for(var b=0,c=0;0<=(b=a.indexOf("//*"))&&(c=a.indexOf("*//"))>=
b;)a=a.substring(0,b)+a.substring(c+3);return a=JU.PT.rep(a,"//","")},"~S");c$.extractFlags=h(c$,"extractFlags",function(a,b){a=JS.SmilesParser.cleanPattern(a);for(var c=0;a.startsWith("/");){var d=JS.SmilesParser.getSubPattern(a,0,"/").toUpperCase();a=a.substring(d.length+2);c=JS.SmilesSearch.addFlags(c,d)}b[0]=c;return a},"~S,~A");c$.getFlags=h(c$,"getFlags",function(a){var b=s(1,0);JS.SmilesParser.extractFlags(a,b);return b[0]},"~S")});B("JS");C(null,"JS.SmilesExt","java.lang.Float JU.AU $.BS $.Lst $.M4 $.Measure $.P3 J.api.Interface JU.Logger".split(" "),
function(){c$=D(function(){this.sm=this.e=null;E(this,arguments)},JS,"SmilesExt");F(c$,function(){});h(c$,"init",function(a){this.e=a;this.sm=this.e.vwr.getSmilesMatcher();return this},"~O");h(c$,"getSmilesCorrelation",function(a,b,c,d,e,f,g,j,h,n,k,m){var s=null==h?0.1:3.4028235E38;try{null==d&&(d=new JU.Lst,e=new JU.Lst);var p=new JU.M4,q=new JU.P3,r=this.e.vwr.ms.at,u=this.e.vwr.ms.ac,y=this.sm.getCorrelationMaps(c,r,u,a,m|8);null==y&&this.e.evalError(this.sm.getLastException(),null);if(0==y.length)return NaN;
var v=y[0];for(a=0;a<v.length;a++)d.addLast(r[v[a]]);y=this.sm.getCorrelationMaps(c,r,u,b,m);null==y&&this.e.evalError(this.sm.getLastException(),null);if(0==y.length)return NaN;JU.Logger.info(y.length+" mappings found");if(k||!j){b=3.4028235E38;c=null;for(a=0;a<y.length;a++){e.clear();for(var x=0;x<y[a].length;x++)e.addLast(r[y[a][x]]);J.api.Interface.getInterface("JU.Eigen",this.e.vwr,"script");var w=1==e.size()?0:JU.Measure.getTransformMatrix4(d,e,p,null);JU.Logger.info("getSmilesCorrelation stddev="+
w);if(null!=g&&w<s){for(var z=new JU.BS,x=0;x<y[a].length;x++)z.set(y[a][x]);g.addLast(z)}w<b&&(c=y[a],null!=f&&f.setM4(p),null!=n&&n.setT(q),b=w)}null!=h&&(h[0]=v,h[1]=c);e.clear();for(a=0;a<c.length;a++)e.addLast(r[c[a]]);return b}for(a=0;a<y.length;a++)for(x=0;x<y[a].length;x++)e.addLast(r[y[a][x]])}catch(A){if(G(A,Exception))this.e.evalError(A.getMessage(),null);else throw A;}return 0},"JU.BS,JU.BS,~S,JU.Lst,JU.Lst,JU.M4,JU.Lst,~B,~A,JU.P3,~B,~N");h(c$,"getSmilesMatches",function(a,b,c,d,e,f,
g){if(0==a.length||a.endsWith("///")||a.equals("H")||a.equals("top")||a.equalsIgnoreCase("NOAROMATIC"))try{return this.e.vwr.getSmilesOpt(c,0,0,e|(a.equals("H")?4096:0)|(a.equals("top")?8192:0)|(a.equalsIgnoreCase("NOAROMATIC")?16:0),a.endsWith("///")?a:null)}catch(j){if(G(j,Exception))this.e.evalError(j.getMessage(),null);else throw j;}var h;if(null==d){d=2==(e&2);try{if(null==b)h=this.sm.getSubstructureSetArray(a,this.e.vwr.ms.at,this.e.vwr.ms.ac,c,null,e);else{var n=this.sm.find(a,b,(d?2:1)|(g?
8:0));if(!f)return!g?n:0==n.length?s(0,0):n[0];var k=new JU.BS;for(f=0;f<n.length;f++)for(var m=n[f],B=m.length;0<=--B;)0<=m[B]&&k.set(m[B]);if(!d)return s(k.cardinality(),0);var p=s(k.cardinality(),0);f=0;for(var q=k.nextSetBit(0);0<=q;q=k.nextSetBit(q+1))p[f++]=q;return p}}catch(r){if(G(r,Exception))return this.e.evalError(r.getMessage(),null),null;throw r;}}else{h=new JU.Lst;k=this.getSmilesCorrelation(d,c,a,null,null,null,h,!1,null,null,!1,e);if(Float.isNaN(k))return f?new JU.BS:K(-1,[]);this.e.showString("RMSD "+
k+" Angstroms");h=h.toArray(Array(h.size()))}if(f){k=new JU.BS;for(f=0;f<h.length;f++)k.or(h[f]);return k}k=new JU.Lst;for(f=0;f<h.length;f++)k.addLast(h[f]);return k},"~S,~S,JU.BS,JU.BS,~N,~B,~B");h(c$,"getFlexFitList",function(a,b,c,d){var e=JU.AU.newInt2(2);this.getSmilesCorrelation(a,b,c,null,null,null,null,!1,e,null,!1,d?2:1);if(null==e[0])return null;a=this.e.vwr.ms.getDihedralMap(e[0]);b=null==a?null:this.e.vwr.ms.getDihedralMap(e[1]);if(null==b||b.length!=a.length)return null;e=N(a.length,
3,0);c=this.e.vwr.ms.at;JS.SmilesExt.getTorsions(c,b,e,0);JS.SmilesExt.getTorsions(c,a,e,1);b=N(6*a.length,0);for(d=c=0;c<a.length;c++){var f=a[c];b[d++]=f[0];b[d++]=f[1];b[d++]=f[2];b[d++]=f[3];b[d++]=e[c][0];b[d++]=e[c][1]}return b},"JU.BS,JU.BS,~S,~B");c$.getTorsions=h(c$,"getTorsions",function(a,b,c,d){for(var e=b.length;0<=--e;){var f=b[e],f=JU.Measure.computeTorsion(a[f[0]],a[f[1]],a[f[2]],a[f[3]],!0);1==d&&(180<f-c[e][0]?f-=360:-180>=f-c[e][0]&&(f+=360));c[e][d]=f}},"~A,~A,~A,~N")})})(Clazz,
Clazz.getClassName,Clazz.newLongArray,Clazz.doubleToByte,Clazz.doubleToInt,Clazz.doubleToLong,Clazz.declarePackage,Clazz.instanceOf,Clazz.load,Clazz.instantialize,Clazz.decorateAsClass,Clazz.floatToInt,Clazz.floatToLong,Clazz.makeConstructor,Clazz.defineEnumConstant,Clazz.exceptionOf,Clazz.newIntArray,Clazz.defineStatics,Clazz.newFloatArray,Clazz.declareType,Clazz.prepareFields,Clazz.superConstructor,Clazz.newByteArray,Clazz.declareInterface,Clazz.p0p,Clazz.pu$h,Clazz.newShortArray,Clazz.innerTypeInstance,
Clazz.isClassDefined,Clazz.prepareCallback,Clazz.newArray,Clazz.castNullAs,Clazz.floatToShort,Clazz.superCall,Clazz.decorateAsType,Clazz.newBooleanArray,Clazz.newCharArray,Clazz.implementOf,Clazz.newDoubleArray,Clazz.overrideConstructor,Clazz.clone,Clazz.doubleToShort,Clazz.getInheritedLevel,Clazz.getParamsType,Clazz.isAF,Clazz.isAB,Clazz.isAI,Clazz.isAS,Clazz.isASS,Clazz.isAP,Clazz.isAFloat,Clazz.isAII,Clazz.isAFF,Clazz.isAFFF,Clazz.tryToSearchAndExecute,Clazz.getStackTrace,Clazz.inheritArgs,Clazz.alert,
Clazz.defineMethod,Clazz.overrideMethod,Clazz.declareAnonymous,Clazz.cloneFinals);
